@@ -1,27 +1,26 @@
-from __future__ import unicode_literals
-
 import os
 import json
 
 import boto
-import boto3
+import boto.dynamodb2
 import boto.iam
 import boto.s3
 import boto.s3.key
 import boto.cloudformation
 from boto.exception import BotoServerError
-import sure  # noqa
+import sure  # noqa # pylint: disable=unused-import
+from freezegun import freeze_time
 
 import pytest
-from moto.core import ACCOUNT_ID
 
 from moto import (
     mock_cloudformation_deprecated,
     mock_s3_deprecated,
+    mock_sns_deprecated,
+    mock_sqs_deprecated,
     mock_route53_deprecated,
     mock_iam_deprecated,
-    mock_dynamodb2,
-    mock_cloudformation,
+    mock_dynamodb2_deprecated,
 )
 from moto.cloudformation import cloudformation_backends
 
@@ -73,14 +72,19 @@ dummy_template4 = {
 dummy_template_json = json.dumps(dummy_template)
 dummy_template_json2 = json.dumps(dummy_template2)
 dummy_template_json3 = json.dumps(dummy_template3)
+dummy_template_json4 = json.dumps(dummy_template4)
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_create_stack():
     conn = boto.connect_cloudformation()
     conn.create_stack("test_stack", template_body=dummy_template_json)
 
     stack = conn.describe_stacks()[0]
+    stack.stack_id.should.contain(
+        "arn:aws:cloudformation:us-east-1:123456789:stack/test_stack/"
+    )
     stack.stack_name.should.equal("test_stack")
     stack.get_template().should.equal(
         {
@@ -96,6 +100,31 @@ def test_create_stack():
     )
 
 
+@mock_cloudformation_deprecated
+def test_create_stack_with_other_region():
+    conn = boto.cloudformation.connect_to_region("us-west-2")
+    conn.create_stack("test_stack", template_body=dummy_template_json)
+
+    stack = conn.describe_stacks()[0]
+    stack.stack_id.should.contain(
+        "arn:aws:cloudformation:us-west-2:123456789:stack/test_stack/"
+    )
+    stack.stack_name.should.equal("test_stack")
+    stack.get_template().should.equal(
+        {
+            "GetTemplateResponse": {
+                "GetTemplateResult": {
+                    "TemplateBody": dummy_template_json,
+                    "ResponseMetadata": {
+                        "RequestId": "2d06e36c-ac1d-11e0-a958-f9382b6eb86bEXAMPLE"
+                    },
+                }
+            }
+        }
+    )
+
+
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 @mock_route53_deprecated
 def test_create_stack_hosted_zone_by_id():
@@ -137,6 +166,7 @@ def test_create_stack_hosted_zone_by_id():
     assert stack.list_resources()
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_creating_stacks_across_regions():
     west1_conn = boto.cloudformation.connect_to_region("us-west-1")
@@ -149,21 +179,75 @@ def test_creating_stacks_across_regions():
     list(west2_conn.describe_stacks()).should.have.length_of(1)
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
+@mock_sns_deprecated
+@mock_sqs_deprecated
 def test_create_stack_with_notification_arn():
+    sqs_conn = boto.connect_sqs()
+    queue = sqs_conn.create_queue("fake-queue", visibility_timeout=3)
+    queue_arn = queue.get_attributes()["QueueArn"]
+
+    sns_conn = boto.connect_sns()
+    topic = sns_conn.create_topic("fake-topic")
+    topic_arn = topic["CreateTopicResponse"]["CreateTopicResult"]["TopicArn"]
+
+    sns_conn.subscribe(topic_arn, "sqs", queue_arn)
+
     conn = boto.connect_cloudformation()
-    conn.create_stack(
-        "test_stack_with_notifications",
-        template_body=dummy_template_json,
-        notification_arns="arn:aws:sns:us-east-1:{}:fake-queue".format(ACCOUNT_ID),
-    )
+    with freeze_time("2015-01-01 12:00:00"):
+        conn.create_stack(
+            "test_stack_with_notifications",
+            template_body=dummy_template_json,
+            notification_arns=topic_arn,
+        )
 
     stack = conn.describe_stacks()[0]
-    [n.value for n in stack.notification_arns].should.contain(
-        "arn:aws:sns:us-east-1:{}:fake-queue".format(ACCOUNT_ID)
-    )
+    [n.value for n in stack.notification_arns].should.contain(topic_arn)
+
+    with freeze_time("2015-01-01 12:00:01"):
+        message = queue.read(1)
+
+    msg = json.loads(message.get_body())
+    msg["Subject"].should.equal("AWS CloudFormation Notification")
+    msg["Message"].should.contain("StackId='{}'\n".format(stack.stack_id))
+    msg["Message"].should.contain("Timestamp='2015-01-01T12:00:00.000Z'\n")
+    msg["Message"].should.contain("LogicalResourceId='test_stack_with_notifications'\n")
+    msg["Message"].should.contain("ResourceStatus='CREATE_IN_PROGRESS'\n")
+    msg["Message"].should.contain("ResourceStatusReason='User Initiated'\n")
+    msg["Message"].should.contain("ResourceType='AWS::CloudFormation::Stack'\n")
+    msg["Message"].should.contain("StackName='test_stack_with_notifications'\n")
+    msg.should.have.key("MessageId")
+    msg.should.have.key("Signature")
+    msg.should.have.key("SignatureVersion")
+    msg.should.have.key("Subject")
+    msg["Timestamp"].should.equal("2015-01-01T12:00:00.000Z")
+    msg["TopicArn"].should.equal(topic_arn)
+    msg.should.have.key("Type")
+    msg.should.have.key("UnsubscribeURL")
+
+    with freeze_time("2015-01-01 12:00:02"):
+        message = queue.read(1)
+
+    msg = json.loads(message.get_body())
+    msg["Message"].should.contain("StackId='{}'\n".format(stack.stack_id))
+    msg["Message"].should.contain("Timestamp='2015-01-01T12:00:00.000Z'\n")
+    msg["Message"].should.contain("LogicalResourceId='test_stack_with_notifications'\n")
+    msg["Message"].should.contain("ResourceStatus='CREATE_COMPLETE'\n")
+    msg["Message"].should.contain("ResourceStatusReason='None'\n")
+    msg["Message"].should.contain("ResourceType='AWS::CloudFormation::Stack'\n")
+    msg["Message"].should.contain("StackName='test_stack_with_notifications'\n")
+    msg.should.have.key("MessageId")
+    msg.should.have.key("Signature")
+    msg.should.have.key("SignatureVersion")
+    msg.should.have.key("Subject")
+    msg["Timestamp"].should.equal("2015-01-01T12:00:00.000Z")
+    msg["TopicArn"].should.equal(topic_arn)
+    msg.should.have.key("Type")
+    msg.should.have.key("UnsubscribeURL")
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 @mock_s3_deprecated
 def test_create_stack_from_s3_url():
@@ -193,6 +277,7 @@ def test_create_stack_from_s3_url():
     )
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_describe_stack_by_name():
     conn = boto.connect_cloudformation()
@@ -202,6 +287,7 @@ def test_describe_stack_by_name():
     stack.stack_name.should.equal("test_stack")
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_describe_stack_by_stack_id():
     conn = boto.connect_cloudformation()
@@ -213,34 +299,21 @@ def test_describe_stack_by_stack_id():
     stack_by_id.stack_name.should.equal("test_stack")
 
 
-@mock_dynamodb2
+@mock_dynamodb2_deprecated
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_delete_stack_dynamo_template():
     conn = boto.connect_cloudformation()
-    dynamodb_client = boto3.client("dynamodb", region_name="us-east-1")
-    conn.create_stack("test_stack", template_body=dummy_template4)
-    table_desc = dynamodb_client.list_tables()
-    len(table_desc.get("TableNames")).should.equal(1)
+    db_conn = boto.dynamodb2.connect_to_region("us-east-1")
+    #
+    conn.create_stack("test_stack", template_body=dummy_template_json4)
+    db_conn.list_tables()["TableNames"].should.have.length_of(1)
+    #
     conn.delete_stack("test_stack")
-    table_desc = dynamodb_client.list_tables()
-    len(table_desc.get("TableNames")).should.equal(0)
-    conn.create_stack("test_stack", template_body=dummy_template4)
-
-
-@mock_dynamodb2
-@mock_cloudformation
-def test_delete_stack_dynamo_template():
-    conn = boto3.client("cloudformation", region_name="us-east-1")
-    dynamodb_client = boto3.client("dynamodb", region_name="us-east-1")
-    conn.create_stack(StackName="test_stack", TemplateBody=json.dumps(dummy_template4))
-    table_desc = dynamodb_client.list_tables()
-    len(table_desc.get("TableNames")).should.equal(1)
-    conn.delete_stack(StackName="test_stack")
-    table_desc = dynamodb_client.list_tables()
-    len(table_desc.get("TableNames")).should.equal(0)
-    conn.create_stack(StackName="test_stack", TemplateBody=json.dumps(dummy_template4))
+    db_conn.list_tables()["TableNames"].should.have.length_of(0)
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_describe_deleted_stack():
     conn = boto.connect_cloudformation()
@@ -255,6 +328,7 @@ def test_describe_deleted_stack():
     stack_by_id.stack_status.should.equal("DELETE_COMPLETE")
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_get_template_by_name():
     conn = boto.connect_cloudformation()
@@ -275,6 +349,7 @@ def test_get_template_by_name():
     )
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_list_stacks():
     conn = boto.connect_cloudformation()
@@ -286,6 +361,7 @@ def test_list_stacks():
     stacks[0].template_description.should.equal("Stack 1")
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_list_stacks_with_filter():
     conn = boto.connect_cloudformation()
@@ -299,6 +375,7 @@ def test_list_stacks_with_filter():
     stacks.should.have.length_of(1)
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_delete_stack_by_name():
     conn = boto.connect_cloudformation()
@@ -309,6 +386,7 @@ def test_delete_stack_by_name():
     conn.describe_stacks().should.have.length_of(0)
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_delete_stack_by_id():
     conn = boto.connect_cloudformation()
@@ -323,6 +401,7 @@ def test_delete_stack_by_id():
     conn.describe_stacks(stack_id).should.have.length_of(1)
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_delete_stack_with_resource_missing_delete_attr():
     conn = boto.connect_cloudformation()
@@ -333,6 +412,7 @@ def test_delete_stack_with_resource_missing_delete_attr():
     conn.describe_stacks().should.have.length_of(0)
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_bad_describe_stack():
     conn = boto.connect_cloudformation()
@@ -340,6 +420,7 @@ def test_bad_describe_stack():
         conn.describe_stacks("bad_stack")
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated()
 def test_cloudformation_params():
     dummy_template = {
@@ -368,6 +449,7 @@ def test_cloudformation_params():
     param.value.should.equal("testing123")
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_cloudformation_params_conditions_and_resources_are_distinct():
     dummy_template = {
@@ -404,6 +486,7 @@ def test_cloudformation_params_conditions_and_resources_are_distinct():
     ]
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_stack_tags():
     conn = boto.connect_cloudformation()
@@ -417,6 +500,7 @@ def test_stack_tags():
     dict(stack.tags).should.equal({"foo": "bar", "baz": "bleh"})
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_update_stack():
     conn = boto.connect_cloudformation()
@@ -440,6 +524,7 @@ def test_update_stack():
     )
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_update_stack_with_previous_template():
     conn = boto.connect_cloudformation()
@@ -462,6 +547,7 @@ def test_update_stack_with_previous_template():
     )
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_update_stack_with_parameters():
     dummy_template = {
@@ -492,6 +578,7 @@ def test_update_stack_with_parameters():
     assert stack.parameters[0].value == "192.168.0.1/16"
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_update_stack_replace_tags():
     conn = boto.connect_cloudformation()
@@ -508,6 +595,7 @@ def test_update_stack_replace_tags():
     dict(stack.tags).should.equal({"foo": "baz"})
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_update_stack_when_rolled_back():
     conn = boto.connect_cloudformation()
@@ -527,6 +615,7 @@ def test_update_stack_when_rolled_back():
     ex.status.should.equal(400)
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_describe_stack_events_shows_create_update_and_delete():
     conn = boto.connect_cloudformation()
@@ -580,6 +669,7 @@ def test_describe_stack_events_shows_create_update_and_delete():
     err.status.should.equal(400)
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_create_stack_lambda_and_dynamodb():
     conn = boto.connect_cloudformation()
@@ -647,6 +737,7 @@ def test_create_stack_lambda_and_dynamodb():
     assert len(resources) == 4
 
 
+# Has boto3 equivalent
 @mock_cloudformation_deprecated
 def test_create_stack_kinesis():
     conn = boto.connect_cloudformation()
