@@ -1,19 +1,32 @@
-from __future__ import unicode_literals
-
+import copy
 import json
 from collections import OrderedDict
 from datetime import datetime, timedelta
+import os
 import pytz
 
 import boto3
 from botocore.exceptions import ClientError
-import sure  # noqa
+import sure  # noqa # pylint: disable=unused-import
 
 import pytest
+from unittest import SkipTest
 
-from moto import mock_cloudformation, mock_s3, mock_sqs, mock_ec2
+from moto import (
+    mock_cloudformation,
+    mock_dynamodb2,
+    mock_s3,
+    mock_sns,
+    mock_sqs,
+    mock_ec2,
+    mock_iam,
+    mock_lambda,
+)
+from moto import settings
 from moto.core import ACCOUNT_ID
-from .test_cloudformation_stack_crud import dummy_template_json2
+from moto.cloudformation import cloudformation_backends
+from .test_cloudformation_stack_crud import dummy_template_json2, dummy_template_json4
+
 from tests import EXAMPLE_AMI_ID
 
 dummy_template = {
@@ -43,6 +56,28 @@ dummy_template3 = {
     },
 }
 
+
+dummy_template_with_parameters = {
+    "AWSTemplateFormatVersion": "2010-09-09",
+    "Description": "A simple CloudFormation template",
+    "Resources": {
+        "Bucket": {
+            "Type": "AWS::S3::Bucket",
+            "Properties": {"BucketName": {"Ref": "Name"}},
+        }
+    },
+    "Parameters": {
+        "Name": {"Type": "String", "Default": "SomeValue"},
+        "Another": {
+            "Type": "String",
+            "Default": "A",
+            "AllowedValues": ["A", "B"],
+            "Description": "Chose A or B",
+        },
+    },
+}
+
+
 dummy_template_yaml = """---
 AWSTemplateFormatVersion: 2010-09-09
 Description: Stack1 with yaml template
@@ -58,6 +93,7 @@ Resources:
           Value: Test tag
         - Key: Name
           Value: Name tag for tests
+    Parameters:
 """
 
 dummy_template_yaml_with_short_form_func = """---
@@ -77,6 +113,26 @@ Resources:
           Value: Name tag for tests
 """
 
+dummy_yaml_template_with_equals = """---
+AWSTemplateFormatVersion: 2010-09-09
+Description: Stack with yaml template
+Conditions:
+  maybe:
+    Fn::Equals: [!Ref enabled, true]
+Parameters:
+  enabled:
+    Type: String
+    AllowedValues:
+     - true
+     - false
+Resources:
+  VPC1:
+    Type: AWS::EC2::VPC
+    Condition: maybe
+    Properties:
+      CidrBlock: 192.168.0.0/16
+"""
+
 dummy_template_yaml_with_ref = """---
 AWSTemplateFormatVersion: 2010-09-09
 Description: Stack1 with yaml template
@@ -101,6 +157,20 @@ Resources:
           Value: !Ref TagName
 """
 
+dummy_empty_template = {
+    "AWSTemplateFormatVersion": "2010-09-09",
+    "Parameters": {},
+    "Resources": {},
+}
+
+dummy_parametrized_template = {
+    "AWSTemplateFormatVersion": "2010-09-09",
+    "Parameters": {
+        "KeyName": {"Description": "A template parameter", "Type": "String"}
+    },
+    "Resources": {},
+}
+
 dummy_update_template = {
     "AWSTemplateFormatVersion": "2010-09-09",
     "Parameters": {
@@ -190,17 +260,42 @@ dummy_template_special_chars_in_description = {
     },
 }
 
+dummy_unknown_template = {
+    "AWSTemplateFormatVersion": "2010-09-09",
+    "Description": "Stack 1",
+    "Resources": {
+        "UnknownResource": {"Type": "AWS::Cloud9::EnvironmentEC2", "Properties": {}},
+    },
+}
+
 dummy_template_json = json.dumps(dummy_template)
 dummy_template_special_chars_in_description_json = json.dumps(
     dummy_template_special_chars_in_description
 )
+dummy_empty_template_json = json.dumps(dummy_empty_template)
+dummy_parametrized_template_json = json.dumps(dummy_parametrized_template)
 dummy_update_template_json = json.dumps(dummy_update_template)
 dummy_output_template_json = json.dumps(dummy_output_template)
 dummy_import_template_json = json.dumps(dummy_import_template)
 dummy_redrive_template_json = json.dumps(dummy_redrive_template)
+dummy_unknown_template_json = json.dumps(dummy_unknown_template)
 
 
 @mock_cloudformation
+@mock_ec2
+def test_create_stack():
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+    cf_conn.create_stack(StackName="test_stack", TemplateBody=dummy_template_json)
+
+    stack = cf_conn.describe_stacks()["Stacks"][0]
+    stack.should.have.key("StackName").equal("test_stack")
+
+    template = cf_conn.get_template(StackName="test_stack")["TemplateBody"]
+    template.should.equal(dummy_template)
+
+
+@mock_cloudformation
+@mock_ec2
 def test_boto3_describe_stack_instances():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
     cf_conn.create_stack_set(
@@ -558,6 +653,43 @@ def test_update_stack_set():
     )
 
 
+@mock_cloudformation
+def test_update_stack_set_with_previous_value():
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+    param = [
+        {"ParameterKey": "TagDescription", "ParameterValue": "StackSetValue"},
+        {"ParameterKey": "TagName", "ParameterValue": "StackSetValue2"},
+    ]
+    param_overrides = [
+        {"ParameterKey": "TagDescription", "ParameterValue": "OverrideValue"},
+        {"ParameterKey": "TagName", "UsePreviousValue": True},
+    ]
+    cf_conn.create_stack_set(
+        StackSetName="test_stack_set",
+        TemplateBody=dummy_template_yaml_with_ref,
+        Parameters=param,
+    )
+    cf_conn.update_stack_set(
+        StackSetName="test_stack_set",
+        TemplateBody=dummy_template_yaml_with_ref,
+        Parameters=param_overrides,
+    )
+    stackset = cf_conn.describe_stack_set(StackSetName="test_stack_set")
+
+    stackset["StackSet"]["Parameters"][0]["ParameterValue"].should.equal(
+        param_overrides[0]["ParameterValue"]
+    )
+    stackset["StackSet"]["Parameters"][1]["ParameterValue"].should.equal(
+        param[1]["ParameterValue"]
+    )
+    stackset["StackSet"]["Parameters"][0]["ParameterKey"].should.equal(
+        param_overrides[0]["ParameterKey"]
+    )
+    stackset["StackSet"]["Parameters"][1]["ParameterKey"].should.equal(
+        param_overrides[1]["ParameterKey"]
+    )
+
+
 @mock_cloudformation
 def test_boto3_list_stack_set_operations():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
@@ -583,12 +715,13 @@ def test_boto3_list_stack_set_operations():
 @mock_cloudformation
 def test_boto3_bad_list_stack_resources():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+
     with pytest.raises(ClientError):
         cf_conn.list_stack_resources(StackName="test_stack_set")
 
 
 @mock_cloudformation
-def test_boto3_delete_stack_set():
+def test_boto3_delete_stack_set_by_name():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
     cf_conn.create_stack_set(
         StackSetName="test_stack_set", TemplateBody=dummy_template_json
@@ -600,6 +733,20 @@ def test_boto3_delete_stack_set():
     ].should.equal("DELETED")
 
 
+@mock_cloudformation
+def test_boto3_delete_stack_set_by_id():
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+    response = cf_conn.create_stack_set(
+        StackSetName="test_stack_set", TemplateBody=dummy_template_json
+    )
+    stack_set_id = response["StackSetId"]
+    cf_conn.delete_stack_set(StackSetName=stack_set_id)
+
+    cf_conn.describe_stack_set(StackSetName="test_stack_set")["StackSet"][
+        "Status"
+    ].should.equal("DELETED")
+
+
 @mock_cloudformation
 def test_boto3_create_stack_set():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
@@ -680,6 +827,19 @@ def test_boto3_describe_stack_set_params():
     ].should.equal(params)
 
 
+@mock_cloudformation
+def test_boto3_describe_stack_set_by_id():
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+    response = cf_conn.create_stack_set(
+        StackSetName="test_stack", TemplateBody=dummy_template_json,
+    )
+
+    stack_set_id = response["StackSetId"]
+    cf_conn.describe_stack_set(StackSetName=stack_set_id)["StackSet"][
+        "TemplateBody"
+    ].should.equal(dummy_template_json)
+
+
 @mock_cloudformation
 def test_boto3_create_stack():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
@@ -690,6 +850,17 @@ def test_boto3_create_stack():
     )
 
 
+@mock_cloudformation
+def test_boto3_create_stack_fail_missing_parameter():
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+
+    with pytest.raises(ClientError, match="Missing parameter KeyName"):
+
+        cf_conn.create_stack(
+            StackName="test_stack", TemplateBody=dummy_parametrized_template_json
+        )
+
+
 @mock_cloudformation
 def test_boto3_create_stack_s3_long_name():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
@@ -743,42 +914,106 @@ def test_get_template_summary():
     s3 = boto3.client("s3", region_name="us-east-1")
     s3_conn = boto3.resource("s3", region_name="us-east-1")
 
+    # json template
     conn = boto3.client("cloudformation", region_name="us-east-1")
     result = conn.get_template_summary(TemplateBody=json.dumps(dummy_template3))
-
     result["ResourceTypes"].should.equal(["AWS::EC2::VPC"])
     result["Version"].should.equal("2010-09-09")
     result["Description"].should.equal("Stack 3")
+    result["Parameters"].should.equal([])
 
+    # existing stack
     conn.create_stack(StackName="test_stack", TemplateBody=json.dumps(dummy_template3))
-
     result = conn.get_template_summary(StackName="test_stack")
-
     result["ResourceTypes"].should.equal(["AWS::EC2::VPC"])
     result["Version"].should.equal("2010-09-09")
     result["Description"].should.equal("Stack 3")
+    result["Parameters"].should.equal([])
 
+    # json template from s3
     s3_conn.create_bucket(Bucket="foobar")
     s3_conn.Object("foobar", "template-key").put(Body=json.dumps(dummy_template3))
-
     key_url = s3.generate_presigned_url(
         ClientMethod="get_object", Params={"Bucket": "foobar", "Key": "template-key"}
     )
-
     conn.create_stack(StackName="stack_from_url", TemplateURL=key_url)
     result = conn.get_template_summary(TemplateURL=key_url)
     result["ResourceTypes"].should.equal(["AWS::EC2::VPC"])
     result["Version"].should.equal("2010-09-09")
     result["Description"].should.equal("Stack 3")
 
+    # yaml template
     conn = boto3.client("cloudformation", region_name="us-east-1")
     result = conn.get_template_summary(TemplateBody=dummy_template_yaml)
-
     result["ResourceTypes"].should.equal(["AWS::EC2::Instance"])
     result["Version"].should.equal("2010-09-09")
     result["Description"].should.equal("Stack1 with yaml template")
 
 
+@mock_cloudformation
+def test_get_template_summary_for_stack_created_by_changeset_execution():
+    conn = boto3.client("cloudformation", region_name="us-east-1")
+    conn.create_change_set(
+        StackName="stack_from_changeset",
+        TemplateBody=json.dumps(dummy_template3),
+        ChangeSetName="test_changeset",
+        ChangeSetType="CREATE",
+    )
+    with pytest.raises(
+        ClientError,
+        match="GetTemplateSummary cannot be called on REVIEW_IN_PROGRESS stacks",
+    ):
+        conn.get_template_summary(StackName="stack_from_changeset")
+    conn.execute_change_set(ChangeSetName="test_changeset")
+    result = conn.get_template_summary(StackName="stack_from_changeset")
+    result["ResourceTypes"].should.equal(["AWS::EC2::VPC"])
+    result["Version"].should.equal("2010-09-09")
+    result["Description"].should.equal("Stack 3")
+
+
+@mock_s3
+@mock_cloudformation
+def test_get_template_summary_for_template_containing_parameters():
+    conn = boto3.client("cloudformation", region_name="us-east-1")
+    conn.create_stack(
+        StackName="test_stack", TemplateBody=json.dumps(dummy_template_with_parameters)
+    )
+    result = conn.get_template_summary(StackName="test_stack")
+    result.should.match_dict(
+        {
+            "Parameters": [
+                {
+                    "ParameterKey": "Name",
+                    "DefaultValue": "SomeValue",
+                    "ParameterType": "String",
+                    "NoEcho": False,
+                    "Description": "",
+                    "ParameterConstraints": {},
+                },
+                {
+                    "ParameterKey": "Another",
+                    "DefaultValue": "A",
+                    "ParameterType": "String",
+                    "NoEcho": False,
+                    "Description": "Chose A or B",
+                    "ParameterConstraints": {"AllowedValues": ["A", "B"]},
+                },
+            ],
+            "Description": "A simple CloudFormation template",
+            "ResourceTypes": ["AWS::S3::Bucket"],
+            "Version": "2010-09-09",
+            # TODO: get_template_summary should support ResourceIdentifierSummaries
+            # "ResourceIdentifierSummaries": [
+            #     {
+            #         "ResourceType": "AWS::S3::Bucket",
+            #         "LogicalResourceIds": ["Bucket"],
+            #         "ResourceIdentifiers": ["BucketName"],
+            #     }
+            # ],
+        }
+    )
+
+
 @mock_cloudformation
 def test_boto3_create_stack_with_ref_yaml():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
@@ -807,20 +1042,74 @@ def test_creating_stacks_across_regions():
     list(west1_cf.stacks.all()).should.have.length_of(1)
     list(west2_cf.stacks.all()).should.have.length_of(1)
 
+    list(west1_cf.stacks.all())[0].stack_id.should.contain(
+        "arn:aws:cloudformation:us-west-1:123456789:stack/test_stack/"
+    )
+    list(west2_cf.stacks.all())[0].stack_id.should.contain(
+        "arn:aws:cloudformation:us-west-2:123456789:stack/test_stack/"
+    )
+
 
 @mock_cloudformation
+@mock_sns
+@mock_sqs
 def test_create_stack_with_notification_arn():
+    sqs = boto3.resource("sqs", region_name="us-east-1")
+    queue = sqs.create_queue(QueueName="fake-queue")
+    queue_arn = queue.attributes["QueueArn"]
+
+    sns = boto3.client("sns", region_name="us-east-1")
+    topic = sns.create_topic(Name="fake-topic")
+    topic_arn = topic["TopicArn"]
+
+    sns.subscribe(TopicArn=topic_arn, Protocol="sqs", Endpoint=queue_arn)
+
     cf = boto3.resource("cloudformation", region_name="us-east-1")
     cf.create_stack(
         StackName="test_stack_with_notifications",
         TemplateBody=dummy_template_json,
-        NotificationARNs=["arn:aws:sns:us-east-1:{}:fake-queue".format(ACCOUNT_ID)],
+        NotificationARNs=[topic_arn],
     )
 
     stack = list(cf.stacks.all())[0]
-    stack.notification_arns.should.contain(
-        "arn:aws:sns:us-east-1:{}:fake-queue".format(ACCOUNT_ID)
-    )
+    stack.notification_arns.should.contain(topic_arn)
+
+    messages = queue.receive_messages()
+    messages.should.have.length_of(1)
+    msg = json.loads(messages[0].body)
+    msg["Subject"].should.equal("AWS CloudFormation Notification")
+    msg["Message"].should.contain("StackId='{}'\n".format(stack.stack_id))
+    msg["Message"].should.contain("LogicalResourceId='test_stack_with_notifications'\n")
+    msg["Message"].should.contain("ResourceStatus='CREATE_IN_PROGRESS'\n")
+    msg["Message"].should.contain("ResourceStatusReason='User Initiated'\n")
+    msg["Message"].should.contain("ResourceType='AWS::CloudFormation::Stack'\n")
+    msg["Message"].should.contain("StackName='test_stack_with_notifications'\n")
+    msg.should.have.key("MessageId")
+    msg.should.have.key("Signature")
+    msg.should.have.key("SignatureVersion")
+    msg.should.have.key("Subject")
+    msg.should.have.key("Timestamp")
+    msg["TopicArn"].should.equal(topic_arn)
+    msg.should.have.key("Type")
+    msg.should.have.key("UnsubscribeURL")
+
+    messages = queue.receive_messages()
+    messages.should.have.length_of(1)
+    msg = json.loads(messages[0].body)
+    msg["Message"].should.contain("StackId='{}'\n".format(stack.stack_id))
+    msg["Message"].should.contain("LogicalResourceId='test_stack_with_notifications'\n")
+    msg["Message"].should.contain("ResourceStatus='CREATE_COMPLETE'\n")
+    msg["Message"].should.contain("ResourceStatusReason='None'\n")
+    msg["Message"].should.contain("ResourceType='AWS::CloudFormation::Stack'\n")
+    msg["Message"].should.contain("StackName='test_stack_with_notifications'\n")
+    msg.should.have.key("MessageId")
+    msg.should.have.key("Signature")
+    msg.should.have.key("SignatureVersion")
+    msg.should.have.key("Subject")
+    msg.should.have.key("Timestamp")
+    msg["TopicArn"].should.equal(topic_arn)
+    msg.should.have.key("Type")
+    msg.should.have.key("UnsubscribeURL")
 
 
 @mock_cloudformation
@@ -854,6 +1143,125 @@ def test_create_stack_from_s3_url():
     )
 
 
+@mock_cloudformation
+def test_boto3_update_stack_fail_missing_new_parameter():
+
+    name = "update_stack_fail_missing_new_parameter"
+
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+
+    cf_conn.create_stack(StackName=name, TemplateBody=dummy_empty_template_json)
+
+    with pytest.raises(ClientError, match="Missing parameter KeyName"):
+
+        cf_conn.update_stack(
+            StackName=name, TemplateBody=dummy_parametrized_template_json
+        )
+
+
+@mock_cloudformation
+def test_boto3_update_stack_fail_update_same_template_body():
+
+    name = "update_stack_with_previous_value"
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+    params = [
+        {"ParameterKey": "TagName", "ParameterValue": "foo"},
+        {"ParameterKey": "TagDescription", "ParameterValue": "bar"},
+    ]
+
+    cf_conn.create_stack(
+        StackName=name, TemplateBody=dummy_template_yaml_with_ref, Parameters=params,
+    )
+
+    with pytest.raises(ClientError) as exp:
+        cf_conn.update_stack(
+            StackName=name,
+            TemplateBody=dummy_template_yaml_with_ref,
+            Parameters=params,
+        )
+    exp_err = exp.value.response.get("Error")
+    exp_metadata = exp.value.response.get("ResponseMetadata")
+
+    exp_err.get("Code").should.equal("ValidationError")
+    exp_err.get("Message").should.equal(f"Stack [{name}] already exists")
+    exp_metadata.get("HTTPStatusCode").should.equal(400)
+
+    cf_conn.update_stack(
+        StackName=name,
+        TemplateBody=dummy_template_yaml_with_ref,
+        Parameters=[
+            {"ParameterKey": "TagName", "ParameterValue": "new_foo"},
+            {"ParameterKey": "TagDescription", "ParameterValue": "new_bar"},
+        ],
+    )
+
+
+@mock_cloudformation
+def test_boto3_update_stack_deleted_resources_can_reference_deleted_parameters():
+
+    name = "update_stack_deleted_resources_can_reference_deleted_parameters"
+
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+
+    template_json = json.dumps(
+        {
+            "AWSTemplateFormatVersion": "2010-09-09",
+            "Parameters": {"TimeoutParameter": {"Default": 61, "Type": "String"}},
+            "Resources": {
+                "Queue": {
+                    "Type": "AWS::SQS::Queue",
+                    "Properties": {"VisibilityTimeout": {"Ref": "TimeoutParameter"}},
+                }
+            },
+        }
+    )
+
+    cf_conn.create_stack(StackName=name, TemplateBody=template_json)
+
+    response = cf_conn.describe_stack_resources(StackName=name)
+    len(response["StackResources"]).should.equal(1)
+
+    cf_conn.update_stack(StackName=name, TemplateBody=dummy_empty_template_json)
+
+    response = cf_conn.describe_stack_resources(StackName=name)
+    len(response["StackResources"]).should.equal(0)
+
+
+@mock_cloudformation
+def test_boto3_update_stack_deleted_resources_can_reference_deleted_resources():
+
+    name = "update_stack_deleted_resources_can_reference_deleted_resources"
+
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+
+    template_json = json.dumps(
+        {
+            "AWSTemplateFormatVersion": "2010-09-09",
+            "Parameters": {"TimeoutParameter": {"Default": 61, "Type": "String"}},
+            "Resources": {
+                "VPC": {
+                    "Type": "AWS::EC2::VPC",
+                    "Properties": {"CidrBlock": "10.0.0.0/16"},
+                },
+                "Subnet": {
+                    "Type": "AWS::EC2::Subnet",
+                    "Properties": {"VpcId": {"Ref": "VPC"}, "CidrBlock": "10.0.0.0/24"},
+                },
+            },
+        }
+    )
+
+    cf_conn.create_stack(StackName=name, TemplateBody=template_json)
+
+    response = cf_conn.describe_stack_resources(StackName=name)
+    len(response["StackResources"]).should.equal(2)
+
+    cf_conn.update_stack(StackName=name, TemplateBody=dummy_empty_template_json)
+
+    response = cf_conn.describe_stack_resources(StackName=name)
+    len(response["StackResources"]).should.equal(0)
+
+
 @mock_cloudformation
 def test_update_stack_with_previous_value():
     name = "update_stack_with_previous_value"
@@ -910,7 +1318,11 @@ def test_update_stack_from_s3_url():
         ClientMethod="get_object", Params={"Bucket": "foobar", "Key": "template-key"}
     )
 
-    cf_conn.update_stack(StackName="update_stack_from_url", TemplateURL=key_url)
+    cf_conn.update_stack(
+        StackName="update_stack_from_url",
+        TemplateURL=key_url,
+        Parameters=[{"ParameterKey": "KeyName", "ParameterValue": "value"}],
+    )
 
     cf_conn.get_template(StackName="update_stack_from_url")[
         "TemplateBody"
@@ -949,6 +1361,7 @@ def test_create_change_set_from_s3_url():
 
 
 @mock_cloudformation
+@mock_ec2
 def test_describe_change_set():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
     cf_conn.create_change_set(
@@ -984,21 +1397,39 @@ def test_describe_change_set():
 
     # Execute change set
     cf_conn.execute_change_set(ChangeSetName="NewChangeSet")
-    # Verify that the changes have been applied
-    stack = cf_conn.describe_change_set(ChangeSetName="NewChangeSet")
-    stack["Changes"].should.have.length_of(1)
 
+    # Verify that the changes have been applied
+    ec2 = boto3.client("ec2", region_name="us-east-1")
+    ec2.describe_instances()["Reservations"].should.have.length_of(1)
+
+    change_set = cf_conn.describe_change_set(ChangeSetName="NewChangeSet")
+    change_set["Changes"].should.have.length_of(1)
+    change_set["ExecutionStatus"].should.equal("EXECUTE_COMPLETE")
+
+    stack = cf_conn.describe_stacks(StackName="NewStack")["Stacks"][0]
+    stack["StackStatus"].should.equal("CREATE_COMPLETE")
+
+    # create another change set to update the stack
     cf_conn.create_change_set(
         StackName="NewStack",
         TemplateBody=dummy_update_template_json,
         ChangeSetName="NewChangeSet2",
         ChangeSetType="UPDATE",
+        Parameters=[{"ParameterKey": "KeyName", "ParameterValue": "value"}],
     )
+
     stack = cf_conn.describe_change_set(ChangeSetName="NewChangeSet2")
     stack["ChangeSetName"].should.equal("NewChangeSet2")
     stack["StackName"].should.equal("NewStack")
     stack["Changes"].should.have.length_of(2)
 
+    # Execute change set
+    cf_conn.execute_change_set(ChangeSetName="NewChangeSet2")
+
+    # Verify that the changes have been applied
+    stack = cf_conn.describe_stacks(StackName="NewStack")["Stacks"][0]
+    stack["StackStatus"].should.equal("UPDATE_COMPLETE")
+
 
 @mock_cloudformation
 @mock_ec2
@@ -1058,6 +1489,37 @@ def test_describe_stack_pagination():
     assert "NextToken" not in resp2.keys()
 
 
+@mock_cloudformation
+def test_describe_stack_resource():
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+    cf_conn.create_stack(StackName="test_stack", TemplateBody=dummy_template_json)
+
+    stack = cf_conn.describe_stacks(StackName="test_stack")["Stacks"][0]
+
+    response = cf_conn.describe_stack_resource(
+        StackName=stack["StackName"], LogicalResourceId="EC2Instance1"
+    )
+
+    resource = response["StackResourceDetail"]
+    resource["LogicalResourceId"].should.equal("EC2Instance1")
+    resource["ResourceStatus"].should.equal("CREATE_COMPLETE")
+    resource["ResourceType"].should.equal("AWS::EC2::Instance")
+    resource["StackId"].should.equal(stack["StackId"])
+
+
+@mock_cloudformation
+def test_describe_stack_resource_when_resource_does_not_exist():
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+    cf_conn.create_stack(StackName="test_stack", TemplateBody=dummy_template_json)
+
+    stack = cf_conn.describe_stacks(StackName="test_stack")["Stacks"][0]
+
+    with pytest.raises(ClientError, match="does not exist for stack"):
+        cf_conn.describe_stack_resource(
+            StackName=stack["StackName"], LogicalResourceId="DoesNotExist"
+        )
+
+
 @mock_cloudformation
 def test_describe_stack_resources():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
@@ -1224,6 +1686,7 @@ def test_describe_updated_stack():
         RoleARN="arn:aws:iam::{}:role/moto".format(ACCOUNT_ID),
         TemplateBody=dummy_update_template_json,
         Tags=[{"Key": "foo", "Value": "baz"}],
+        Parameters=[{"ParameterKey": "KeyName", "ParameterValue": "value"}],
     )
 
     stack = cf_conn.describe_stacks(StackName="test_stack")["Stacks"][0]
@@ -1235,15 +1698,39 @@ def test_describe_updated_stack():
     stack_by_id["RoleARN"].should.equal("arn:aws:iam::{}:role/moto".format(ACCOUNT_ID))
     stack_by_id["Tags"].should.equal([{"Key": "foo", "Value": "baz"}])
 
+    # Verify the updated template is persisted
+    template = cf_conn.get_template(StackName="test_stack")["TemplateBody"]
+    template.should.equal(dummy_update_template)
+
+
+@mock_cloudformation
+def test_update_stack_with_previous_template():
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+    cf_conn.create_stack(StackName="test_stack", TemplateBody=dummy_template_json)
+    cf_conn.update_stack(StackName="test_stack", UsePreviousTemplate=True)
+
+    stack = cf_conn.describe_stacks(StackName="test_stack")["Stacks"][0]
+    stack["StackName"].should.equal("test_stack")
+    stack["StackStatus"].should.equal("UPDATE_COMPLETE")
+
+    # Verify the original template is persisted
+    template = cf_conn.get_template(StackName="test_stack")["TemplateBody"]
+    template.should.equal(dummy_template)
+
 
 @mock_cloudformation
 def test_bad_describe_stack():
     cf_conn = boto3.client("cloudformation", region_name="us-east-1")
-    with pytest.raises(ClientError):
+    with pytest.raises(ClientError) as exc:
         cf_conn.describe_stacks(StackName="non_existent_stack")
+    err = exc.value.response["Error"]
+    err.should.have.key("Code").being.equal("ValidationError")
+    err.should.have.key("Message").being.equal(
+        "Stack with id non_existent_stack does not exist"
+    )
 
 
-@mock_cloudformation()
+@mock_cloudformation
 def test_cloudformation_params():
     dummy_template_with_params = {
         "AWSTemplateFormatVersion": "2010-09-09",
@@ -1272,6 +1759,154 @@ def test_cloudformation_params():
     param["ParameterValue"].should.equal("testing123")
 
 
+@mock_cloudformation
+@mock_ec2
+def test_update_stack_with_parameters():
+    template = {
+        "AWSTemplateFormatVersion": "2010-09-09",
+        "Description": "Stack",
+        "Resources": {
+            "VPC": {
+                "Properties": {"CidrBlock": {"Ref": "Bar"}},
+                "Type": "AWS::EC2::VPC",
+            }
+        },
+        "Parameters": {"Bar": {"Type": "String"}},
+    }
+    template_json = json.dumps(template)
+    cf = boto3.client("cloudformation", region_name="us-east-1")
+    cf.create_stack(
+        StackName="test_stack",
+        TemplateBody=template_json,
+        Parameters=[{"ParameterKey": "Bar", "ParameterValue": "192.168.0.0/16"}],
+    )
+    cf.update_stack(
+        StackName="test_stack",
+        TemplateBody=template_json,
+        Parameters=[{"ParameterKey": "Bar", "ParameterValue": "192.168.0.1/16"}],
+    )
+
+    stack = cf.describe_stacks(StackName="test_stack")["Stacks"][0]
+    stack["Parameters"].should.have.length_of(1)
+    stack["Parameters"][0].should.equal(
+        {"ParameterKey": "Bar", "ParameterValue": "192.168.0.1/16"}
+    )
+
+
+@mock_cloudformation
+@mock_ec2
+def test_update_stack_replace_tags():
+    cf = boto3.client("cloudformation", region_name="us-east-1")
+    cf.create_stack(
+        StackName="test_stack",
+        TemplateBody=dummy_template_json,
+        Tags=[{"Key": "foo", "Value": "bar"}],
+    )
+    cf.update_stack(
+        StackName="test_stack",
+        TemplateBody=dummy_template_json,
+        Tags=[{"Key": "foo", "Value": "baz"}],
+    )
+
+    stack = cf.describe_stacks(StackName="test_stack")["Stacks"][0]
+    stack["StackStatus"].should.equal("UPDATE_COMPLETE")
+    stack["Tags"].should.equal([{"Key": "foo", "Value": "baz"}])
+
+
+@mock_cloudformation
+def test_update_stack_when_rolled_back():
+    if settings.TEST_SERVER_MODE:
+        raise SkipTest("Cant manipulate backend in server mode")
+    cf = boto3.client("cloudformation", region_name="us-east-1")
+    stack = cf.create_stack(StackName="test_stack", TemplateBody=dummy_template_json)
+    stack_id = stack["StackId"]
+
+    cloudformation_backends["us-east-1"].stacks[stack_id].status = "ROLLBACK_COMPLETE"
+
+    with pytest.raises(ClientError) as ex:
+        cf.update_stack(StackName="test_stack", TemplateBody=dummy_template_json)
+
+    err = ex.value.response["Error"]
+    err.should.have.key("Code").being.equal("ValidationError")
+    err.should.have.key("Message").match(
+        r"Stack:arn:aws:cloudformation:us-east-1:123456789:stack/test_stack/[a-z0-9-]+ is in ROLLBACK_COMPLETE state and can not be updated."
+    )
+
+
+@mock_cloudformation
+@mock_ec2
+def test_cloudformation_params_conditions_and_resources_are_distinct():
+    template_with_conditions = {
+        "AWSTemplateFormatVersion": "2010-09-09",
+        "Description": "Stack 1",
+        "Conditions": {
+            "FooEnabled": {"Fn::Equals": [{"Ref": "FooEnabled"}, "true"]},
+            "FooDisabled": {
+                "Fn::Not": [{"Fn::Equals": [{"Ref": "FooEnabled"}, "true"]}]
+            },
+        },
+        "Parameters": {
+            "FooEnabled": {"Type": "String", "AllowedValues": ["true", "false"]}
+        },
+        "Resources": {
+            "Bar": {
+                "Properties": {"CidrBlock": "192.168.0.0/16"},
+                "Condition": "FooDisabled",
+                "Type": "AWS::EC2::VPC",
+            }
+        },
+    }
+    template_with_conditions = json.dumps(template_with_conditions)
+    cf = boto3.client("cloudformation", region_name="us-east-1")
+    cf.create_stack(
+        StackName="test_stack1",
+        TemplateBody=template_with_conditions,
+        Parameters=[{"ParameterKey": "FooEnabled", "ParameterValue": "true"}],
+    )
+    resources = cf.list_stack_resources(StackName="test_stack1")[
+        "StackResourceSummaries"
+    ]
+    assert not [
+        resource for resource in resources if resource["LogicalResourceId"] == "Bar"
+    ]
+
+
+@mock_cloudformation
+@mock_ec2
+def test_cloudformation_conditions_yaml_equals():
+    cf = boto3.client("cloudformation", region_name="us-east-1")
+    cf.create_stack(
+        StackName="teststack2",
+        TemplateBody=dummy_yaml_template_with_equals,
+        Parameters=[{"ParameterKey": "enabled", "ParameterValue": "true"}],
+    )
+    resources = cf.list_stack_resources(StackName="teststack2")[
+        "StackResourceSummaries"
+    ]
+    assert [
+        resource for resource in resources if resource["LogicalResourceId"] == "VPC1"
+    ]
+
+
+@mock_cloudformation
+@mock_ec2
+def test_cloudformation_conditions_yaml_equals_shortform():
+    _template = dummy_yaml_template_with_equals
+    _template = _template.replace("Fn::Equals:", "!Equals")
+    cf = boto3.client("cloudformation", region_name="us-east-1")
+    cf.create_stack(
+        StackName="teststack2",
+        TemplateBody=_template,
+        Parameters=[{"ParameterKey": "enabled", "ParameterValue": "true"}],
+    )
+    resources = cf.list_stack_resources(StackName="teststack2")[
+        "StackResourceSummaries"
+    ]
+    assert [
+        resource for resource in resources if resource["LogicalResourceId"] == "VPC1"
+    ]
+
+
 @mock_cloudformation
 def test_stack_tags():
     tags = [{"Key": "foo", "Value": "bar"}, {"Key": "baz", "Value": "bleh"}]
@@ -1293,7 +1928,10 @@ def test_stack_tags():
 def test_stack_events():
     cf = boto3.resource("cloudformation", region_name="us-east-1")
     stack = cf.create_stack(StackName="test_stack", TemplateBody=dummy_template_json)
-    stack.update(TemplateBody=dummy_update_template_json)
+    stack.update(
+        TemplateBody=dummy_update_template_json,
+        Parameters=[{"ParameterKey": "KeyName", "ParameterValue": "value"}],
+    )
     stack = cf.Stack(stack.stack_id)
     stack.delete()
 
@@ -1449,3 +2087,141 @@ def test_boto3_create_duplicate_stack():
         cf_conn.create_stack(
             StackName="test_stack", TemplateBody=dummy_template_json,
         )
+
+
+@mock_dynamodb2
+@mock_cloudformation
+def test_delete_stack_dynamo_template():
+    conn = boto3.client("cloudformation", region_name="us-east-1")
+    dynamodb_client = boto3.client("dynamodb", region_name="us-east-1")
+    conn.create_stack(StackName="test_stack", TemplateBody=dummy_template_json4)
+    table_desc = dynamodb_client.list_tables()
+    len(table_desc.get("TableNames")).should.equal(1)
+    conn.delete_stack(StackName="test_stack")
+    table_desc = dynamodb_client.list_tables()
+    len(table_desc.get("TableNames")).should.equal(0)
+    conn.create_stack(StackName="test_stack", TemplateBody=dummy_template_json4)
+
+
+@mock_dynamodb2
+@mock_cloudformation
+@mock_lambda
+def test_create_stack_lambda_and_dynamodb():
+    if settings.TEST_SERVER_MODE:
+        raise SkipTest("Cant set environment variables in server mode")
+    cf = boto3.client("cloudformation", region_name="us-east-1")
+    template = {
+        "AWSTemplateFormatVersion": "2010-09-09",
+        "Description": "Stack Lambda Test 1",
+        "Parameters": {},
+        "Resources": {
+            "func1": {
+                "Type": "AWS::Lambda::Function",
+                "Properties": {
+                    "Code": {"S3Bucket": "bucket_123", "S3Key": "key_123"},
+                    "FunctionName": "func1",
+                    "Handler": "handler.handler",
+                    "Role": get_role_name(),
+                    "Runtime": "python2.7",
+                    "Description": "descr",
+                    "MemorySize": 12345,
+                },
+            },
+            "func1version": {
+                "Type": "AWS::Lambda::Version",
+                "Properties": {"FunctionName": {"Ref": "func1"}},
+            },
+            "tab1": {
+                "Type": "AWS::DynamoDB::Table",
+                "Properties": {
+                    "TableName": "tab1",
+                    "KeySchema": [{"AttributeName": "attr1", "KeyType": "HASH"}],
+                    "AttributeDefinitions": [
+                        {"AttributeName": "attr1", "AttributeType": "string"}
+                    ],
+                    "ProvisionedThroughput": {
+                        "ReadCapacityUnits": 10,
+                        "WriteCapacityUnits": 10,
+                    },
+                    "StreamSpecification": {"StreamViewType": "KEYS_ONLY"},
+                },
+            },
+            "func1mapping": {
+                "Type": "AWS::Lambda::EventSourceMapping",
+                "Properties": {
+                    "FunctionName": {"Ref": "func1"},
+                    "EventSourceArn": {"Fn::GetAtt": ["tab1", "StreamArn"]},
+                    "StartingPosition": "0",
+                    "BatchSize": 100,
+                    "Enabled": True,
+                },
+            },
+        },
+    }
+    validate_s3_before = os.environ.get("VALIDATE_LAMBDA_S3", "")
+    try:
+        os.environ["VALIDATE_LAMBDA_S3"] = "false"
+        cf.create_stack(
+            StackName="test_stack_lambda", TemplateBody=json.dumps(template),
+        )
+    finally:
+        os.environ["VALIDATE_LAMBDA_S3"] = validate_s3_before
+
+    resources = cf.list_stack_resources(StackName="test_stack_lambda")[
+        "StackResourceSummaries"
+    ]
+    resources.should.have.length_of(4)
+    resource_types = [r["ResourceType"] for r in resources]
+    resource_types.should.contain("AWS::Lambda::Function")
+    resource_types.should.contain("AWS::Lambda::Version")
+    resource_types.should.contain("AWS::DynamoDB::Table")
+    resource_types.should.contain("AWS::Lambda::EventSourceMapping")
+
+
+@mock_cloudformation
+@mock_ec2
+def test_create_and_update_stack_with_unknown_resource():
+    cf_conn = boto3.client("cloudformation", region_name="us-east-1")
+    # Creating a stack with an unknown resource should throw a warning
+    expected_err = "Tried to parse AWS::Cloud9::EnvironmentEC2 but it's not supported by moto's CloudFormation implementation"
+    if settings.TEST_SERVER_MODE:
+        # Can't verify warnings in ServerMode though
+        cf_conn.create_stack(
+            StackName="test_stack", TemplateBody=dummy_unknown_template_json
+        )
+    else:
+        with pytest.warns(UserWarning, match=expected_err):
+            cf_conn.create_stack(
+                StackName="test_stack", TemplateBody=dummy_unknown_template_json
+            )
+
+    # The stack should exist though
+    stacks = cf_conn.describe_stacks()["Stacks"]
+    stacks.should.have.length_of(1)
+    stacks[0].should.have.key("StackName").equal("test_stack")
+
+    # Updating an unknown resource should throw a warning, but not fail
+    new_template = copy.deepcopy(dummy_unknown_template)
+    new_template["Resources"]["UnknownResource"]["Properties"]["Sth"] = "other"
+    if settings.TEST_SERVER_MODE:
+        cf_conn.update_stack(
+            StackName="test_stack", TemplateBody=json.dumps(new_template)
+        )
+    else:
+        with pytest.warns(UserWarning, match=expected_err):
+            cf_conn.update_stack(
+                StackName="test_stack", TemplateBody=json.dumps(new_template)
+            )
+
+
+def get_role_name():
+    with mock_iam():
+        iam = boto3.client("iam", region_name="us-east-1")
+        try:
+            return iam.get_role(RoleName="my-role")["Role"]["Arn"]
+        except ClientError:
+            return iam.create_role(
+                RoleName="my-role",
+                AssumeRolePolicyDocument="some policy",
+                Path="/my-path/",
+            )["Role"]["Arn"]
