@@ -1,17 +1,19 @@
-from __future__ import unicode_literals
+import boto3
 import json
 import yaml
 
-from mock import patch
-import sure  # noqa
+import pytest
+import sure  # noqa # pylint: disable=unused-import
+from botocore.exceptions import ClientError
+from unittest.mock import patch
 
 from moto.cloudformation.exceptions import ValidationError
 from moto.cloudformation.models import FakeStack
 from moto.cloudformation.parsing import (
     resource_class_from_type,
     parse_condition,
-    Export,
 )
+from moto import mock_cloudformation, mock_sqs, mock_ssm, settings
 from moto.sqs.models import Queue
 from moto.s3.models import FakeBucket
 from moto.cloudformation.utils import yaml_tag_constructor
@@ -20,7 +22,7 @@ from moto.packages.boto.cloudformation.stack import Output
 
 dummy_template = {
     "AWSTemplateFormatVersion": "2010-09-09",
-    "Description": "Create a multi-az, load balanced, Auto Scaled sample web site. The Auto Scaling trigger is based on the CPU utilization of the web servers. The AMI is chosen based on the region in which the stack is run. This example creates a web service running across all availability zones in a region. The instances are load balanced with a simple health check. The web site is available on port 80, however, the instances can be configured to listen on any port (8888 by default). **WARNING** This template creates one or more Amazon EC2 instances. You will be billed for the AWS resources used if you create a stack from this template.",
+    "Description": "sample template",
     "Resources": {
         "Queue": {
             "Type": "AWS::SQS::Queue",
@@ -32,7 +34,7 @@ dummy_template = {
 
 name_type_template = {
     "AWSTemplateFormatVersion": "2010-09-09",
-    "Description": "Create a multi-az, load balanced, Auto Scaled sample web site. The Auto Scaling trigger is based on the CPU utilization of the web servers. The AMI is chosen based on the region in which the stack is run. This example creates a web service running across all availability zones in a region. The instances are load balanced with a simple health check. The web site is available on port 80, however, the instances can be configured to listen on any port (8888 by default). **WARNING** This template creates one or more Amazon EC2 instances. You will be billed for the AWS resources used if you create a stack from this template.",
+    "Description": "sample template",
     "Resources": {
         "Queue": {"Type": "AWS::SQS::Queue", "Properties": {"VisibilityTimeout": 60}}
     },
@@ -41,7 +43,7 @@ name_type_template = {
 name_type_template_with_tabs_json = """
 \t{
 \t\t"AWSTemplateFormatVersion": "2010-09-09",
-\t\t"Description": "Create a multi-az, load balanced, Auto Scaled sample web site. The Auto Scaling trigger is based on the CPU utilization of the web servers. The AMI is chosen based on the region in which the stack is run. This example creates a web service running across all availability zones in a region. The instances are load balanced with a simple health check. The web site is available on port 80, however, the instances can be configured to listen on any port (8888 by default). **WARNING** This template creates one or more Amazon EC2 instances. You will be billed for the AWS resources used if you create a stack from this template.",
+\t\t"Description": "sample template",
 \t\t"Resources": {
 \t\t\t"Queue": {"Type": "AWS::SQS::Queue", "Properties": {"VisibilityTimeout": 60}}
 \t\t}
@@ -54,6 +56,8 @@ output_dict = {
     }
 }
 
+null_output = {"Outputs": None}
+
 bad_output = {
     "Outputs": {"Output1": {"Value": {"Fn::GetAtt": ["Queue", "InvalidAttribute"]}}}
 }
@@ -73,6 +77,16 @@ parameters = {
     }
 }
 
+ssm_parameter = {
+    "Parameters": {
+        "SingleParamCfn": {"Type": "AWS::SSM::Parameter::Value<String>"},
+        "ListParamCfn": {
+            "Type": "AWS::SSM::Parameter::Value<List<String>>",
+            "Default": "/path/to/list/param",
+        },
+    }
+}
+
 split_select_template = {
     "AWSTemplateFormatVersion": "2010-09-09",
     "Resources": {
@@ -134,6 +148,7 @@ import_value_template = {
 }
 
 outputs_template = dict(list(dummy_template.items()) + list(output_dict.items()))
+null_outputs_template = dict(list(dummy_template.items()) + list(null_output.items()))
 bad_outputs_template = dict(list(dummy_template.items()) + list(bad_output.items()))
 get_attribute_outputs_template = dict(
     list(dummy_template.items()) + list(get_attribute_output.items())
@@ -143,14 +158,19 @@ get_availability_zones_template = dict(
 )
 
 parameters_template = dict(list(dummy_template.items()) + list(parameters.items()))
+ssm_parameter_template = dict(
+    list(dummy_template.items()) + list(ssm_parameter.items())
+)
 
 dummy_template_json = json.dumps(dummy_template)
 name_type_template_json = json.dumps(name_type_template)
 output_type_template_json = json.dumps(outputs_template)
+null_output_template_json = json.dumps(null_outputs_template)
 bad_output_template_json = json.dumps(bad_outputs_template)
 get_attribute_outputs_template_json = json.dumps(get_attribute_outputs_template)
 get_availability_zones_template_json = json.dumps(get_availability_zones_template)
 parameters_template_json = json.dumps(parameters_template)
+ssm_parameter_template_json = json.dumps(ssm_parameter_template)
 split_select_template_json = json.dumps(split_select_template)
 sub_template_json = json.dumps(sub_template)
 export_value_template_json = json.dumps(export_value_template)
@@ -294,10 +314,25 @@ def test_parse_stack_with_get_availability_zones():
     output.value.should.equal(["us-east-1a", "us-east-1b", "us-east-1c", "us-east-1d"])
 
 
-def test_parse_stack_with_bad_get_attribute_outputs():
+@mock_sqs
+@mock_cloudformation
+def test_parse_stack_with_bad_get_attribute_outputs_using_boto3():
+    conn = boto3.client("cloudformation", region_name="us-west-1")
+    with pytest.raises(ClientError) as exc:
+        conn.create_stack(StackName="teststack", TemplateBody=bad_output_template_json)
+    err = exc.value.response["Error"]
+    err["Code"].should.equal("ValidationError")
+    err["Message"].should.equal(
+        "Template error: resource Queue does not support attribute type InvalidAttribute in Fn::GetAtt"
+    )
+
+
+def test_parse_stack_with_null_outputs_section():
     FakeStack.when.called_with(
-        "test_id", "test_stack", bad_output_template_json, {}, "us-west-1"
-    ).should.throw(ValidationError)
+        "test_id", "test_stack", null_output_template_json, {}, "us-west-1"
+    ).should.throw(
+        ValidationError, "[/Outputs] 'null' values are not allowed in templates"
+    )
 
 
 def test_parse_stack_with_parameters():
@@ -498,3 +533,44 @@ def test_short_form_func_in_yaml_teamplate():
     ]
     for k, v in key_and_expects:
         template_dict.should.have.key(k).which.should.be.equal(v)
+
+
+@mock_ssm
+def test_ssm_parameter_parsing():
+    client = boto3.client("ssm", region_name="us-west-1")
+    client.put_parameter(Name="/path/to/single/param", Value="string", Type="String")
+    client.put_parameter(
+        Name="/path/to/list/param", Value="comma,separated,string", Type="StringList"
+    )
+
+    if not settings.TEST_SERVER_MODE:
+        stack = FakeStack(
+            stack_id="test_id",
+            name="test_stack",
+            template=ssm_parameter_template_json,
+            parameters={
+                "SingleParamCfn": "/path/to/single/param",
+                "ListParamCfn": "/path/to/list/param",
+            },
+            region_name="us-west-1",
+        )
+
+        stack.resource_map.resolved_parameters["SingleParamCfn"].should.equal("string")
+        stack.resource_map.resolved_parameters["ListParamCfn"].should.equal(
+            ["comma", "separated", "string"]
+        )
+
+    # Not passing in a value for ListParamCfn to test Default value
+    if not settings.TEST_SERVER_MODE:
+        stack = FakeStack(
+            stack_id="test_id",
+            name="test_stack",
+            template=ssm_parameter_template_json,
+            parameters={"SingleParamCfn": "/path/to/single/param",},
+            region_name="us-west-1",
+        )
+
+        stack.resource_map.resolved_parameters["SingleParamCfn"].should.equal("string")
+        stack.resource_map.resolved_parameters["ListParamCfn"].should.equal(
+            ["comma", "separated", "string"]
+        )
