@@ -7,7 +7,6 @@ from moto.dynamodb2.exceptions import (
     ExpressionAttributeNameNotDefined,
     IncorrectOperandType,
     InvalidUpdateExpressionInvalidDocumentPath,
-    EmptyKeyAttributeException,
 )
 from moto.dynamodb2.models import Item, DynamoType
 from moto.dynamodb2.parsing.ast_nodes import (
@@ -19,25 +18,25 @@ from moto.dynamodb2.parsing.expressions import UpdateExpressionParser
 from moto.dynamodb2.parsing.validators import UpdateExpressionValidator
 
 
-def test_validation_of_empty_string_key_val(table):
-    with pytest.raises(EmptyKeyAttributeException):
-        update_expression = "set forum_name=:NewName"
-        update_expression_values = {":NewName": {"S": ""}}
-        update_expression_ast = UpdateExpressionParser.make(update_expression)
-        item = Item(
-            hash_key=DynamoType({"S": "forum_name"}),
-            hash_key_type="TYPE",
-            range_key=None,
-            range_key_type=None,
-            attrs={"forum_name": {"S": "hello"}},
-        )
-        UpdateExpressionValidator(
-            update_expression_ast,
-            expression_attribute_names=None,
-            expression_attribute_values=update_expression_values,
-            item=item,
-            table=table,
-        ).validate()
+def test_valid_update_expression(table):
+    update_expression = "set forum_desc=:Desc, forum_type=:NewType"
+    update_expression_values = {
+        ":Desc": {"S": "AmazingForum"},
+        ":NewType": {"S": "BASIC"},
+    }
+    update_expression_ast = UpdateExpressionParser.make(update_expression)
+    item = Item(
+        hash_key=DynamoType({"S": "forum_name"}),
+        range_key=DynamoType({"S": "forum_type"}),
+        attrs={"forum_name": {"S": "hello"}},
+    )
+    UpdateExpressionValidator(
+        update_expression_ast,
+        expression_attribute_names=None,
+        expression_attribute_values=update_expression_values,
+        item=item,
+        table=table,
+    ).validate()
 
 
 def test_validation_of_update_expression_with_keyword(table):
@@ -47,9 +46,7 @@ def test_validation_of_update_expression_with_keyword(table):
         update_expression_ast = UpdateExpressionParser.make(update_expression)
         item = Item(
             hash_key=DynamoType({"S": "id"}),
-            hash_key_type="TYPE",
             range_key=None,
-            range_key_type=None,
             attrs={"id": {"S": "1"}, "path": {"N": "3"}},
         )
         UpdateExpressionValidator(
@@ -65,7 +62,7 @@ def test_validation_of_update_expression_with_keyword(table):
 
 
 @pytest.mark.parametrize(
-    "update_expression", ["SET a = #b + :val2", "SET a = :val2 + #b",]
+    "update_expression", ["SET a = #b + :val2", "SET a = :val2 + #b",],
 )
 def test_validation_of_a_set_statement_with_incorrect_passed_value(
     update_expression, table
@@ -79,9 +76,7 @@ def test_validation_of_a_set_statement_with_incorrect_passed_value(
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
         hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
         range_key=None,
-        range_key_type=None,
         attrs={"id": {"S": "1"}, "b": {"N": "3"}},
     )
     try:
@@ -110,9 +105,7 @@ def test_validation_of_update_expression_with_attribute_that_does_not_exist_in_i
         update_expression_ast = UpdateExpressionParser.make(update_expression)
         item = Item(
             hash_key=DynamoType({"S": "id"}),
-            hash_key_type="TYPE",
             range_key=None,
-            range_key_type=None,
             attrs={"id": {"S": "1"}, "path": {"N": "3"}},
         )
         UpdateExpressionValidator(
@@ -127,7 +120,9 @@ def test_validation_of_update_expression_with_attribute_that_does_not_exist_in_i
         assert True
 
 
-@pytest.mark.parametrize("update_expression", ["SET a = #c", "SET a = #c + #d",])
+@pytest.mark.parametrize(
+    "update_expression", ["SET a = #c", "SET a = #c + #d",],
+)
 def test_validation_of_update_expression_with_attribute_name_that_is_not_defined(
     update_expression, table,
 ):
@@ -141,9 +136,7 @@ def test_validation_of_update_expression_with_attribute_name_that_is_not_defined
         update_expression_ast = UpdateExpressionParser.make(update_expression)
         item = Item(
             hash_key=DynamoType({"S": "id"}),
-            hash_key_type="TYPE",
             range_key=None,
-            range_key_type=None,
             attrs={"id": {"S": "1"}, "path": {"N": "3"}},
         )
         UpdateExpressionValidator(
@@ -164,9 +157,7 @@ def test_validation_of_if_not_exists_not_existing_invalid_replace_value(table):
         update_expression_ast = UpdateExpressionParser.make(update_expression)
         item = Item(
             hash_key=DynamoType({"S": "id"}),
-            hash_key_type="TYPE",
             range_key=None,
-            range_key_type=None,
             attrs={"id": {"S": "1"}, "a": {"S": "A"}},
         )
         UpdateExpressionValidator(
@@ -208,9 +199,7 @@ def test_validation_of_if_not_exists_not_existing_value(table):
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
         hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
         range_key=None,
-        range_key_type=None,
         attrs={"id": {"S": "1"}, "a": {"S": "A"}},
     )
     validated_ast = UpdateExpressionValidator(
@@ -231,9 +220,7 @@ def test_validation_of_if_not_exists_with_existing_attribute_should_return_attri
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
         hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
         range_key=None,
-        range_key_type=None,
         attrs={"id": {"S": "1"}, "a": {"S": "A"}, "b": {"S": "B"}},
     )
     validated_ast = UpdateExpressionValidator(
@@ -253,9 +240,7 @@ def test_validation_of_if_not_exists_with_existing_attribute_should_return_value
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
         hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
         range_key=None,
-        range_key_type=None,
         attrs={"id": {"S": "1"}, "b": {"N": "3"}},
     )
     validated_ast = UpdateExpressionValidator(
@@ -276,11 +261,7 @@ def test_validation_of_if_not_exists_with_non_existing_attribute_should_return_v
     update_expression_values = {":val": {"N": "4"}}
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
-        hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
-        range_key=None,
-        range_key_type=None,
-        attrs={"id": {"S": "1"}},
+        hash_key=DynamoType({"S": "id"}), range_key=None, attrs={"id": {"S": "1"}},
     )
     validated_ast = UpdateExpressionValidator(
         update_expression_ast,
@@ -298,9 +279,7 @@ def test_validation_of_sum_operation(table):
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
         hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
         range_key=None,
-        range_key_type=None,
         attrs={"id": {"S": "1"}, "a": {"N": "3"}, "b": {"N": "4"}},
     )
     validated_ast = UpdateExpressionValidator(
@@ -319,9 +298,7 @@ def test_validation_homogeneous_list_append_function(table):
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
         hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
         range_key=None,
-        range_key_type=None,
         attrs={"id": {"S": "1"}, "ri": {"L": [{"S": "i1"}, {"S": "i2"}]}},
     )
     validated_ast = UpdateExpressionValidator(
@@ -342,9 +319,7 @@ def test_validation_hetereogenous_list_append_function(table):
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
         hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
         range_key=None,
-        range_key_type=None,
         attrs={"id": {"S": "1"}, "ri": {"L": [{"S": "i1"}, {"S": "i2"}]}},
     )
     validated_ast = UpdateExpressionValidator(
@@ -371,9 +346,7 @@ def test_validation_list_append_function_with_non_list_arg(table):
         update_expression_ast = UpdateExpressionParser.make(update_expression)
         item = Item(
             hash_key=DynamoType({"S": "id"}),
-            hash_key_type="TYPE",
             range_key=None,
-            range_key_type=None,
             attrs={"id": {"S": "1"}, "ri": {"L": [{"S": "i1"}, {"S": "i2"}]}},
         )
         UpdateExpressionValidator(
@@ -400,9 +373,7 @@ def test_sum_with_incompatible_types(table):
         update_expression_ast = UpdateExpressionParser.make(update_expression)
         item = Item(
             hash_key=DynamoType({"S": "id"}),
-            hash_key_type="TYPE",
             range_key=None,
-            range_key_type=None,
             attrs={"id": {"S": "1"}, "ri": {"L": [{"S": "i1"}, {"S": "i2"}]}},
         )
         UpdateExpressionValidator(
@@ -422,9 +393,7 @@ def test_validation_of_subraction_operation(table):
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
         hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
         range_key=None,
-        range_key_type=None,
         attrs={"id": {"S": "1"}, "a": {"N": "3"}, "b": {"N": "4"}},
     )
     validated_ast = UpdateExpressionValidator(
@@ -448,9 +417,7 @@ def test_cannot_index_into_a_string(table):
         update_expression_ast = UpdateExpressionParser.make(update_expression)
         item = Item(
             hash_key=DynamoType({"S": "id"}),
-            hash_key_type="TYPE",
             range_key=None,
-            range_key_type=None,
             attrs={"id": {"S": "foo2"}, "itemstr": {"S": "somestring"}},
         )
         UpdateExpressionValidator(
@@ -473,9 +440,7 @@ def test_validation_set_path_does_not_need_to_be_resolvable_when_setting_a_new_a
     update_expression_ast = UpdateExpressionParser.make(update_expression)
     item = Item(
         hash_key=DynamoType({"S": "id"}),
-        hash_key_type="TYPE",
         range_key=None,
-        range_key_type=None,
         attrs={"id": {"S": "foo2"}, "a": {"N": "3"}},
     )
     validated_ast = UpdateExpressionValidator(
@@ -497,9 +462,7 @@ def test_validation_set_path_does_not_need_to_be_resolvable_but_must_be_creatabl
         update_expression_ast = UpdateExpressionParser.make(update_expression)
         item = Item(
             hash_key=DynamoType({"S": "id"}),
-            hash_key_type="TYPE",
             range_key=None,
-            range_key_type=None,
             attrs={"id": {"S": "foo2"}, "a": {"N": "3"}},
         )
         UpdateExpressionValidator(
