@@ -1,10 +1,8 @@
-from __future__ import unicode_literals
-
 import json
-from six.moves.urllib.parse import quote
+from urllib.parse import quote
 
 import pytest
-import sure  # noqa
+import sure  # noqa # pylint: disable=unused-import
 
 import moto.server as server
 from moto import mock_iot
