@@ -1,11 +1,18 @@
-from __future__ import unicode_literals
-
 from datetime import datetime
 
+from moto.organizations.exceptions import InvalidInputException, TargetNotFoundException
+from moto.organizations.models import (
+    FakeAccount,
+    FakeOrganization,
+    FakeOrganizationalUnit,
+    FakePolicy,
+    FakeRoot,
+    OrganizationsBackend,
+)
+
 import boto3
 import json
-import six
-import sure  # noqa
+import sure  # noqa # pylint: disable=unused-import
 from botocore.exceptions import ClientError
 import pytest
 
@@ -62,7 +69,7 @@ def test_describe_organization():
 def test_describe_organization_exception():
     client = boto3.client("organizations", region_name="us-east-1")
     with pytest.raises(ClientError) as e:
-        response = client.describe_organization()
+        client.describe_organization()
     ex = e.value
     ex.operation_name.should.equal("DescribeOrganization")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -109,9 +116,9 @@ def test_describe_organizational_unit():
 @mock_organizations
 def test_describe_organizational_unit_exception():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     with pytest.raises(ClientError) as e:
-        response = client.describe_organizational_unit(
+        client.describe_organizational_unit(
             OrganizationalUnitId=utils.make_random_root_id()
         )
     ex = e.value
@@ -140,7 +147,7 @@ def test_list_organizational_units_for_parent():
 def test_list_organizational_units_for_parent_exception():
     client = boto3.client("organizations", region_name="us-east-1")
     with pytest.raises(ClientError) as e:
-        response = client.list_organizational_units_for_parent(
+        client.list_organizational_units_for_parent(
             ParentId=utils.make_random_root_id()
         )
     ex = e.value
@@ -194,7 +201,7 @@ def test_describe_account():
 def test_describe_account_exception():
     client = boto3.client("organizations", region_name="us-east-1")
     with pytest.raises(ClientError) as e:
-        response = client.describe_account(AccountId=utils.make_random_account_id())
+        client.describe_account(AccountId=utils.make_random_account_id())
     ex = e.value
     ex.operation_name.should.equal("DescribeAccount")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -225,7 +232,7 @@ def test_list_accounts():
 @mock_organizations
 def test_list_accounts_for_parent():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     account_id = client.create_account(AccountName=mockname, Email=mockemail)[
         "CreateAccountStatus"
@@ -237,7 +244,7 @@ def test_list_accounts_for_parent():
 @mock_organizations
 def test_move_account():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     account_id = client.create_account(AccountName=mockname, Email=mockemail)[
         "CreateAccountStatus"
@@ -254,7 +261,7 @@ def test_move_account():
 @mock_organizations
 def test_list_parents_for_ou():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     ou01 = client.create_organizational_unit(ParentId=root_id, Name="ou01")
     ou01_id = ou01["OrganizationalUnit"]["Id"]
@@ -273,7 +280,7 @@ def test_list_parents_for_ou():
 @mock_organizations
 def test_list_parents_for_accounts():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     ou01 = client.create_organizational_unit(ParentId=root_id, Name="ou01")
     ou01_id = ou01["OrganizationalUnit"]["Id"]
@@ -299,7 +306,7 @@ def test_list_parents_for_accounts():
 @mock_organizations
 def test_list_children():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     ou01 = client.create_organizational_unit(ParentId=root_id, Name="ou01")
     ou01_id = ou01["OrganizationalUnit"]["Id"]
@@ -333,18 +340,16 @@ def test_list_children():
 @mock_organizations
 def test_list_children_exception():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     with pytest.raises(ClientError) as e:
-        response = client.list_children(
-            ParentId=utils.make_random_root_id(), ChildType="ACCOUNT"
-        )
+        client.list_children(ParentId=utils.make_random_root_id(), ChildType="ACCOUNT")
     ex = e.value
     ex.operation_name.should.equal("ListChildren")
     ex.response["Error"]["Code"].should.equal("400")
     ex.response["Error"]["Message"].should.contain("ParentNotFoundException")
     with pytest.raises(ClientError) as e:
-        response = client.list_children(ParentId=root_id, ChildType="BLEE")
+        client.list_children(ParentId=root_id, ChildType="BLEE")
     ex = e.value
     ex.operation_name.should.equal("ListChildren")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -361,9 +366,9 @@ def test_list_create_account_status():
     createAccountStatuses.should.have.length_of(1)
     validate_create_account_status(createAccountStatuses[0])
 
-    request_id = client.create_account(AccountName=mockname, Email=mockemail)[
-        "CreateAccountStatus"
-    ]["Id"]
+    client.create_account(AccountName=mockname, Email=mockemail)["CreateAccountStatus"][
+        "Id"
+    ]
     response = client.list_create_account_status()
     createAccountStatuses = response["CreateAccountStatuses"]
     createAccountStatuses.should.have.length_of(2)
@@ -396,8 +401,8 @@ def test_list_create_account_status_in_progress():
 def test_get_paginated_list_create_account_status():
     client = boto3.client("organizations", region_name="us-east-1")
     client.create_organization(FeatureSet="ALL")["Organization"]
-    for i in range(5):
-        request_id = client.create_account(AccountName=mockname, Email=mockemail)[
+    for _ in range(5):
+        client.create_account(AccountName=mockname, Email=mockemail)[
             "CreateAccountStatus"
         ]["Id"]
     response = client.list_create_account_status(MaxResults=2)
@@ -491,13 +496,13 @@ def test_describe_policy_exception():
     client.create_organization(FeatureSet="ALL")["Organization"]
     policy_id = "p-47fhe9s3"
     with pytest.raises(ClientError) as e:
-        response = client.describe_policy(PolicyId=policy_id)
+        client.describe_policy(PolicyId=policy_id)
     ex = e.value
     ex.operation_name.should.equal("DescribePolicy")
     ex.response["Error"]["Code"].should.equal("400")
     ex.response["Error"]["Message"].should.contain("PolicyNotFoundException")
     with pytest.raises(ClientError) as e:
-        response = client.describe_policy(PolicyId="meaninglessstring")
+        client.describe_policy(PolicyId="meaninglessstring")
     ex = e.value
     ex.operation_name.should.equal("DescribePolicy")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -508,7 +513,7 @@ def test_describe_policy_exception():
 @mock_organizations
 def test_attach_policy():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     ou_id = client.create_organizational_unit(ParentId=root_id, Name="ou01")[
         "OrganizationalUnit"
@@ -533,7 +538,7 @@ def test_attach_policy():
 @mock_organizations
 def test_detach_policy():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     ou_id = client.create_organizational_unit(ParentId=root_id, Name="ou01")[
         "OrganizationalUnit"
@@ -547,23 +552,38 @@ def test_detach_policy():
         Name="MockServiceControlPolicy",
         Type="SERVICE_CONTROL_POLICY",
     )["Policy"]["PolicySummary"]["Id"]
-    client.attach_policy(PolicyId=policy_id, TargetId=ou_id)
-    client.attach_policy(PolicyId=policy_id, TargetId=root_id)
-    client.attach_policy(PolicyId=policy_id, TargetId=account_id)
-    response = client.detach_policy(PolicyId=policy_id, TargetId=ou_id)
-    response["ResponseMetadata"]["HTTPStatusCode"].should.equal(200)
-    response = client.detach_policy(PolicyId=policy_id, TargetId=root_id)
-    response["ResponseMetadata"]["HTTPStatusCode"].should.equal(200)
-    response = client.detach_policy(PolicyId=policy_id, TargetId=account_id)
-    response["ResponseMetadata"]["HTTPStatusCode"].should.equal(200)
+    # Attach/List/Detach policy
+    for name, target in [("OU", ou_id), ("Root", root_id), ("Account", account_id)]:
+        #
+        with sure.ensure("We should start with 0 policies"):
+            get_nonaws_policies(target, client).should.have.length_of(0)
+        #
+        client.attach_policy(PolicyId=policy_id, TargetId=target)
+        with sure.ensure("Expecting 1 policy after creation of target={0}", name):
+            get_nonaws_policies(target, client).should.have.length_of(1)
+        #
+        response = client.detach_policy(PolicyId=policy_id, TargetId=target)
+        response["ResponseMetadata"]["HTTPStatusCode"].should.equal(200)
+        with sure.ensure("Expecting 0 policies after deletion of target={0}", name):
+            get_nonaws_policies(target, client).should.have.length_of(0)
+
+
+def get_nonaws_policies(account_id, client):
+    return [
+        p
+        for p in client.list_policies_for_target(
+            TargetId=account_id, Filter="SERVICE_CONTROL_POLICY"
+        )["Policies"]
+        if not p["AwsManaged"]
+    ]
 
 
 @mock_organizations
 def test_detach_policy_root_ou_not_found_exception():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
-    ou_id = client.create_organizational_unit(ParentId=root_id, Name="ou01")[
+    client.create_organizational_unit(ParentId=root_id, Name="ou01")[
         "OrganizationalUnit"
     ]["Id"]
     account_id = client.create_account(AccountName=mockname, Email=mockemail)[
@@ -578,7 +598,7 @@ def test_detach_policy_root_ou_not_found_exception():
     client.attach_policy(PolicyId=policy_id, TargetId=root_id)
     client.attach_policy(PolicyId=policy_id, TargetId=account_id)
     with pytest.raises(ClientError) as e:
-        response = client.detach_policy(PolicyId=policy_id, TargetId="r-xy85")
+        client.detach_policy(PolicyId=policy_id, TargetId="r-xy85")
     ex = e.value
     ex.operation_name.should.equal("DetachPolicy")
     ex.response["Error"]["Code"].should.equal("400")
@@ -590,7 +610,7 @@ def test_detach_policy_root_ou_not_found_exception():
 @mock_organizations
 def test_detach_policy_ou_not_found_exception():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     ou_id = client.create_organizational_unit(ParentId=root_id, Name="ou01")[
         "OrganizationalUnit"
@@ -603,9 +623,7 @@ def test_detach_policy_ou_not_found_exception():
     )["Policy"]["PolicySummary"]["Id"]
     client.attach_policy(PolicyId=policy_id, TargetId=ou_id)
     with pytest.raises(ClientError) as e:
-        response = client.detach_policy(
-            PolicyId=policy_id, TargetId="ou-zx86-z3x4yr2t7"
-        )
+        client.detach_policy(PolicyId=policy_id, TargetId="ou-zx86-z3x4yr2t7")
     ex = e.value
     ex.operation_name.should.equal("DetachPolicy")
     ex.response["Error"]["Code"].should.equal("400")
@@ -617,7 +635,7 @@ def test_detach_policy_ou_not_found_exception():
 @mock_organizations
 def test_detach_policy_account_id_not_found_exception():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     account_id = client.create_account(AccountName=mockname, Email=mockemail)[
         "CreateAccountStatus"
     ]["AccountId"]
@@ -629,7 +647,7 @@ def test_detach_policy_account_id_not_found_exception():
     )["Policy"]["PolicySummary"]["Id"]
     client.attach_policy(PolicyId=policy_id, TargetId=account_id)
     with pytest.raises(ClientError) as e:
-        response = client.detach_policy(PolicyId=policy_id, TargetId="111619863336")
+        client.detach_policy(PolicyId=policy_id, TargetId="111619863336")
     ex = e.value
     ex.operation_name.should.equal("DetachPolicy")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -642,7 +660,7 @@ def test_detach_policy_account_id_not_found_exception():
 @mock_organizations
 def test_detach_policy_invalid_target_exception():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     ou_id = client.create_organizational_unit(ParentId=root_id, Name="ou01")[
         "OrganizationalUnit"
@@ -655,7 +673,7 @@ def test_detach_policy_invalid_target_exception():
     )["Policy"]["PolicySummary"]["Id"]
     client.attach_policy(PolicyId=policy_id, TargetId=ou_id)
     with pytest.raises(ClientError) as e:
-        response = client.detach_policy(PolicyId=policy_id, TargetId="invalidtargetid")
+        client.detach_policy(PolicyId=policy_id, TargetId="invalidtargetid")
     ex = e.value
     ex.operation_name.should.equal("DetachPolicy")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -666,7 +684,7 @@ def test_detach_policy_invalid_target_exception():
 @mock_organizations
 def test_delete_policy():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     base_policies = client.list_policies(Filter="SERVICE_CONTROL_POLICY")["Policies"]
     base_policies.should.have.length_of(1)
     policy_id = client.create_policy(
@@ -687,10 +705,10 @@ def test_delete_policy():
 @mock_organizations
 def test_delete_policy_exception():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     non_existent_policy_id = utils.make_random_policy_id()
     with pytest.raises(ClientError) as e:
-        response = client.delete_policy(PolicyId=non_existent_policy_id)
+        client.delete_policy(PolicyId=non_existent_policy_id)
     ex = e.value
     ex.operation_name.should.equal("DeletePolicy")
     ex.response["Error"]["Code"].should.equal("400")
@@ -706,7 +724,7 @@ def test_delete_policy_exception():
     root_id = client.list_roots()["Roots"][0]["Id"]
     client.attach_policy(PolicyId=policy_id, TargetId=root_id)
     with pytest.raises(ClientError) as e:
-        response = client.delete_policy(PolicyId=policy_id)
+        client.delete_policy(PolicyId=policy_id)
     ex = e.value
     ex.operation_name.should.equal("DeletePolicy")
     ex.response["Error"]["Code"].should.equal("400")
@@ -727,7 +745,7 @@ def test_attach_policy_exception():
         Type="SERVICE_CONTROL_POLICY",
     )["Policy"]["PolicySummary"]["Id"]
     with pytest.raises(ClientError) as e:
-        response = client.attach_policy(PolicyId=policy_id, TargetId=root_id)
+        client.attach_policy(PolicyId=policy_id, TargetId=root_id)
     ex = e.value
     ex.operation_name.should.equal("AttachPolicy")
     ex.response["Error"]["Code"].should.equal("400")
@@ -735,7 +753,7 @@ def test_attach_policy_exception():
         "OrganizationalUnitNotFoundException"
     )
     with pytest.raises(ClientError) as e:
-        response = client.attach_policy(PolicyId=policy_id, TargetId=ou_id)
+        client.attach_policy(PolicyId=policy_id, TargetId=ou_id)
     ex = e.value
     ex.operation_name.should.equal("AttachPolicy")
     ex.response["Error"]["Code"].should.equal("400")
@@ -743,7 +761,7 @@ def test_attach_policy_exception():
         "OrganizationalUnitNotFoundException"
     )
     with pytest.raises(ClientError) as e:
-        response = client.attach_policy(PolicyId=policy_id, TargetId=account_id)
+        client.attach_policy(PolicyId=policy_id, TargetId=account_id)
     ex = e.value
     ex.operation_name.should.equal("AttachPolicy")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -752,9 +770,7 @@ def test_attach_policy_exception():
         "You specified an account that doesn't exist."
     )
     with pytest.raises(ClientError) as e:
-        response = client.attach_policy(
-            PolicyId=policy_id, TargetId="meaninglessstring"
-        )
+        client.attach_policy(PolicyId=policy_id, TargetId="meaninglessstring")
     ex = e.value
     ex.operation_name.should.equal("AttachPolicy")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -790,10 +806,10 @@ def test_update_policy():
 @mock_organizations
 def test_update_policy_exception():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     non_existent_policy_id = utils.make_random_policy_id()
     with pytest.raises(ClientError) as e:
-        response = client.update_policy(PolicyId=non_existent_policy_id)
+        client.update_policy(PolicyId=non_existent_policy_id)
     ex = e.value
     ex.operation_name.should.equal("UpdatePolicy")
     ex.response["Error"]["Code"].should.equal("400")
@@ -855,9 +871,7 @@ def test_list_policies_for_target_exception():
     ou_id = "ou-gi99-i7r8eh2i2"
     account_id = "126644886543"
     with pytest.raises(ClientError) as e:
-        response = client.list_policies_for_target(
-            TargetId=ou_id, Filter="SERVICE_CONTROL_POLICY"
-        )
+        client.list_policies_for_target(TargetId=ou_id, Filter="SERVICE_CONTROL_POLICY")
     ex = e.value
     ex.operation_name.should.equal("ListPoliciesForTarget")
     ex.response["Error"]["Code"].should.equal("400")
@@ -865,7 +879,7 @@ def test_list_policies_for_target_exception():
         "OrganizationalUnitNotFoundException"
     )
     with pytest.raises(ClientError) as e:
-        response = client.list_policies_for_target(
+        client.list_policies_for_target(
             TargetId=account_id, Filter="SERVICE_CONTROL_POLICY"
         )
     ex = e.value
@@ -876,7 +890,7 @@ def test_list_policies_for_target_exception():
         "You specified an account that doesn't exist."
     )
     with pytest.raises(ClientError) as e:
-        response = client.list_policies_for_target(
+        client.list_policies_for_target(
             TargetId="meaninglessstring", Filter="SERVICE_CONTROL_POLICY"
         )
     ex = e.value
@@ -917,7 +931,7 @@ def test_list_policies_for_target_exception():
 @mock_organizations
 def test_list_targets_for_policy():
     client = boto3.client("organizations", region_name="us-east-1")
-    org = client.create_organization(FeatureSet="ALL")["Organization"]
+    client.create_organization(FeatureSet="ALL")["Organization"]
     root_id = client.list_roots()["Roots"][0]["Id"]
     ou_id = client.create_organizational_unit(ParentId=root_id, Name="ou01")[
         "OrganizationalUnit"
@@ -937,9 +951,9 @@ def test_list_targets_for_policy():
     response = client.list_targets_for_policy(PolicyId=policy_id)
     for target in response["Targets"]:
         target.should.be.a(dict)
-        target.should.have.key("Name").should.be.a(six.string_types)
-        target.should.have.key("Arn").should.be.a(six.string_types)
-        target.should.have.key("TargetId").should.be.a(six.string_types)
+        target.should.have.key("Name").should.be.a(str)
+        target.should.have.key("Arn").should.be.a(str)
+        target.should.have.key("TargetId").should.be.a(str)
         target.should.have.key("Type").should.be.within(
             ["ROOT", "ORGANIZATIONAL_UNIT", "ACCOUNT"]
         )
@@ -951,13 +965,13 @@ def test_list_targets_for_policy_exception():
     client.create_organization(FeatureSet="ALL")["Organization"]
     policy_id = "p-47fhe9s3"
     with pytest.raises(ClientError) as e:
-        response = client.list_targets_for_policy(PolicyId=policy_id)
+        client.list_targets_for_policy(PolicyId=policy_id)
     ex = e.value
     ex.operation_name.should.equal("ListTargetsForPolicy")
     ex.response["Error"]["Code"].should.equal("400")
     ex.response["Error"]["Message"].should.contain("PolicyNotFoundException")
     with pytest.raises(ClientError) as e:
-        response = client.list_targets_for_policy(PolicyId="meaninglessstring")
+        client.list_targets_for_policy(PolicyId="meaninglessstring")
     ex = e.value
     ex.operation_name.should.equal("ListTargetsForPolicy")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -966,26 +980,84 @@ def test_list_targets_for_policy_exception():
 
 
 @mock_organizations
-def test_tag_resource():
+def test_tag_resource_account():
     client = boto3.client("organizations", region_name="us-east-1")
     client.create_organization(FeatureSet="ALL")
-    account_id = client.create_account(AccountName=mockname, Email=mockemail)[
+    resource_id = client.create_account(AccountName=mockname, Email=mockemail)[
         "CreateAccountStatus"
     ]["AccountId"]
 
-    client.tag_resource(ResourceId=account_id, Tags=[{"Key": "key", "Value": "value"}])
+    client.tag_resource(ResourceId=resource_id, Tags=[{"Key": "key", "Value": "value"}])
 
-    response = client.list_tags_for_resource(ResourceId=account_id)
+    response = client.list_tags_for_resource(ResourceId=resource_id)
     response["Tags"].should.equal([{"Key": "key", "Value": "value"}])
 
     # adding a tag with an existing key, will update the value
     client.tag_resource(
-        ResourceId=account_id, Tags=[{"Key": "key", "Value": "new-value"}]
+        ResourceId=resource_id, Tags=[{"Key": "key", "Value": "new-value"}]
     )
 
-    response = client.list_tags_for_resource(ResourceId=account_id)
+    response = client.list_tags_for_resource(ResourceId=resource_id)
     response["Tags"].should.equal([{"Key": "key", "Value": "new-value"}])
 
+    client.untag_resource(ResourceId=resource_id, TagKeys=["key"])
+
+    response = client.list_tags_for_resource(ResourceId=resource_id)
+    response["Tags"].should.equal([])
+
+
+@mock_organizations
+def test_tag_resource_organization_organization_root():
+    client = boto3.client("organizations", region_name="us-east-1")
+    client.create_organization(FeatureSet="ALL")
+
+    resource_id = client.list_roots()["Roots"][0]["Id"]
+    client.tag_resource(ResourceId=resource_id, Tags=[{"Key": "key", "Value": "value"}])
+
+    response = client.list_tags_for_resource(ResourceId=resource_id)
+    response["Tags"].should.equal([{"Key": "key", "Value": "value"}])
+
+    # adding a tag with an existing key, will update the value
+    client.tag_resource(
+        ResourceId=resource_id, Tags=[{"Key": "key", "Value": "new-value"}]
+    )
+
+    response = client.list_tags_for_resource(ResourceId=resource_id)
+    response["Tags"].should.equal([{"Key": "key", "Value": "new-value"}])
+
+    client.untag_resource(ResourceId=resource_id, TagKeys=["key"])
+
+    response = client.list_tags_for_resource(ResourceId=resource_id)
+    response["Tags"].should.equal([])
+
+
+@mock_organizations
+def test_tag_resource_organization_organizational_unit():
+    client = boto3.client("organizations", region_name="us-east-1")
+    client.create_organization(FeatureSet="ALL")
+    root_id = client.list_roots()["Roots"][0]["Id"]
+    resource_id = client.create_organizational_unit(ParentId=root_id, Name="ou01")[
+        "OrganizationalUnit"
+    ]["Id"]
+
+    client.tag_resource(ResourceId=resource_id, Tags=[{"Key": "key", "Value": "value"}])
+
+    response = client.list_tags_for_resource(ResourceId=resource_id)
+    response["Tags"].should.equal([{"Key": "key", "Value": "value"}])
+
+    # adding a tag with an existing key, will update the value
+    client.tag_resource(
+        ResourceId=resource_id, Tags=[{"Key": "key", "Value": "new-value"}]
+    )
+
+    response = client.list_tags_for_resource(ResourceId=resource_id)
+    response["Tags"].should.equal([{"Key": "key", "Value": "new-value"}])
+
+    client.untag_resource(ResourceId=resource_id, TagKeys=["key"])
+
+    response = client.list_tags_for_resource(ResourceId=resource_id)
+    response["Tags"].should.equal([])
+
 
 @mock_organizations
 def test_tag_resource_errors():
@@ -994,7 +1066,7 @@ def test_tag_resource_errors():
 
     with pytest.raises(ClientError) as e:
         client.tag_resource(
-            ResourceId="000000000000", Tags=[{"Key": "key", "Value": "value"},],
+            ResourceId="0A000000X000", Tags=[{"Key": "key", "Value": "value"},],
         )
     ex = e.value
     ex.operation_name.should.equal("TagResource")
@@ -1003,6 +1075,109 @@ def test_tag_resource_errors():
     ex.response["Error"]["Message"].should.equal(
         "You provided a value that does not match the required pattern."
     )
+    with pytest.raises(ClientError) as e:
+        client.tag_resource(
+            ResourceId="000000000000", Tags=[{"Key": "key", "Value": "value"}]
+        )
+    ex = e.value
+    ex.operation_name.should.equal("TagResource")
+    ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
+    ex.response["Error"]["Code"].should.contain("TargetNotFoundException")
+    ex.response["Error"]["Message"].should.equal(
+        "You specified a target that doesn't exist."
+    )
+
+
+def test__get_resource_for_tagging_existing_root():
+    org = FakeOrganization("ALL")
+    root = FakeRoot(org)
+
+    org_backend = OrganizationsBackend()
+    org_backend.ou.append(root)
+    response = org_backend._get_resource_for_tagging(root.id)
+    response.id.should.equal(root.id)
+
+
+def test__get_resource_for_tagging_existing_non_root():
+    org_backend = OrganizationsBackend()
+    with pytest.raises(TargetNotFoundException) as e:
+        org_backend._get_resource_for_tagging("r-abcd")
+    ex = e.value
+    ex.code.should.equal(400)
+    ex.description.should.contain("TargetNotFoundException")
+    ex.message.should.equal("You specified a target that doesn't exist.")
+
+
+def test__get_resource_for_tagging_existing_ou():
+    org = FakeOrganization("ALL")
+    ou = FakeOrganizationalUnit(org)
+    org_backend = OrganizationsBackend()
+
+    org_backend.ou.append(ou)
+    response = org_backend._get_resource_for_tagging(ou.id)
+    response.id.should.equal(ou.id)
+
+
+def test__get_resource_for_tagging_non_existing_ou():
+    org_backend = OrganizationsBackend()
+    with pytest.raises(TargetNotFoundException) as e:
+        org_backend._get_resource_for_tagging("ou-9oyc-lv2q36ln")
+    ex = e.value
+    ex.code.should.equal(400)
+    ex.description.should.contain("TargetNotFoundException")
+    ex.message.should.equal("You specified a target that doesn't exist.")
+
+
+def test__get_resource_for_tagging_existing_account():
+    org = FakeOrganization("ALL")
+    org_backend = OrganizationsBackend()
+    account = FakeAccount(org, AccountName="test", Email="test@test.test")
+
+    org_backend.accounts.append(account)
+    response = org_backend._get_resource_for_tagging(account.id)
+    response.id.should.equal(account.id)
+
+
+def test__get_resource_for_tagging_non_existing_account():
+    org_backend = OrganizationsBackend()
+    with pytest.raises(TargetNotFoundException) as e:
+        org_backend._get_resource_for_tagging("100326223992")
+    ex = e.value
+    ex.code.should.equal(400)
+    ex.description.should.contain("TargetNotFoundException")
+    ex.message.should.equal("You specified a target that doesn't exist.")
+
+
+def test__get_resource_for_tagging_existing_policy():
+    org = FakeOrganization("ALL")
+    org_backend = OrganizationsBackend()
+    policy = FakePolicy(org, Type="SERVICE_CONTROL_POLICY")
+
+    org_backend.policies.append(policy)
+    response = org_backend._get_resource_for_tagging(policy.id)
+    response.id.should.equal(policy.id)
+
+
+def test__get_resource_for_tagging_non_existing_policy():
+    org_backend = OrganizationsBackend()
+    with pytest.raises(TargetNotFoundException) as e:
+        org_backend._get_resource_for_tagging("p-y1vas4da")
+    ex = e.value
+    ex.code.should.equal(400)
+    ex.description.should.contain("TargetNotFoundException")
+    ex.message.should.equal("You specified a target that doesn't exist.")
+
+
+def test__get_resource_to_tag_incorrect_resource():
+    org_backend = OrganizationsBackend()
+    with pytest.raises(InvalidInputException) as e:
+        org_backend._get_resource_for_tagging("10032622399200")
+    ex = e.value
+    ex.code.should.equal(400)
+    ex.description.should.contain("InvalidInputException")
+    ex.message.should.equal(
+        "You provided a value that does not match the required pattern."
+    )
 
 
 @mock_organizations
@@ -1025,7 +1200,7 @@ def test_list_tags_for_resource_errors():
     client.create_organization(FeatureSet="ALL")
 
     with pytest.raises(ClientError) as e:
-        client.list_tags_for_resource(ResourceId="000000000000")
+        client.list_tags_for_resource(ResourceId="000x00000A00")
     ex = e.value
     ex.operation_name.should.equal("ListTagsForResource")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -1033,6 +1208,15 @@ def test_list_tags_for_resource_errors():
     ex.response["Error"]["Message"].should.equal(
         "You provided a value that does not match the required pattern."
     )
+    with pytest.raises(ClientError) as e:
+        client.list_tags_for_resource(ResourceId="000000000000")
+    ex = e.value
+    ex.operation_name.should.equal("ListTagsForResource")
+    ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
+    ex.response["Error"]["Code"].should.contain("TargetNotFoundException")
+    ex.response["Error"]["Message"].should.equal(
+        "You specified a target that doesn't exist."
+    )
 
 
 @mock_organizations
@@ -1062,7 +1246,7 @@ def test_untag_resource_errors():
     client.create_organization(FeatureSet="ALL")
 
     with pytest.raises(ClientError) as e:
-        client.untag_resource(ResourceId="000000000000", TagKeys=["key"])
+        client.untag_resource(ResourceId="0X00000000A0", TagKeys=["key"])
     ex = e.value
     ex.operation_name.should.equal("UntagResource")
     ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
@@ -1070,6 +1254,15 @@ def test_untag_resource_errors():
     ex.response["Error"]["Message"].should.equal(
         "You provided a value that does not match the required pattern."
     )
+    with pytest.raises(ClientError) as e:
+        client.untag_resource(ResourceId="000000000000", TagKeys=["key"])
+    ex = e.value
+    ex.operation_name.should.equal("UntagResource")
+    ex.response["ResponseMetadata"]["HTTPStatusCode"].should.equal(400)
+    ex.response["Error"]["Code"].should.contain("TargetNotFoundException")
+    ex.response["Error"]["Message"].should.equal(
+        "You specified a target that doesn't exist."
+    )
 
 
 @mock_organizations
@@ -1125,7 +1318,7 @@ def test_enable_aws_service_access():
     service = response["EnabledServicePrincipals"][0]
     service["ServicePrincipal"].should.equal("config.amazonaws.com")
     date_enabled = service["DateEnabled"]
-    date_enabled["DateEnabled"].should.be.a(datetime)
+    date_enabled.should.be.a(datetime)
 
     # enabling the same service again should not result in any error or change
     # when
@@ -1140,7 +1333,7 @@ def test_enable_aws_service_access():
 
 
 @mock_organizations
-def test_enable_aws_service_access():
+def test_enable_aws_service_access_error():
     client = boto3.client("organizations", region_name="us-east-1")
     client.create_organization(FeatureSet="ALL")
 
@@ -1156,7 +1349,7 @@ def test_enable_aws_service_access():
 
 
 @mock_organizations
-def test_enable_aws_service_access():
+def test_enable_multiple_aws_service_access():
     # given
     client = boto3.client("organizations", region_name="us-east-1")
     client.create_organization(FeatureSet="ALL")
