@@ -1,8 +1,5 @@
 # #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-import mock
-
 from moto.packages.httpretty.core import (
     HTTPrettyRequest,
     fake_gethostname,
