@@ -1,23 +1,16 @@
-from __future__ import unicode_literals
-
 import boto3
 import json
 import os
-import sure  # noqa
-import sys
+import sure  # noqa # pylint: disable=unused-import
 from datetime import datetime
 from dateutil.tz import tzutc
 from botocore.exceptions import ClientError
 import pytest
 
-from moto import mock_cloudformation, mock_sts, mock_stepfunctions
+from moto import mock_sts, mock_stepfunctions
 from moto.core import ACCOUNT_ID
 
-if sys.version_info[0] < 3:
-    import mock
-    from unittest import SkipTest
-else:
-    from unittest import SkipTest, mock
+from unittest import SkipTest, mock
 
 region = "us-east-1"
 simple_definition = (
@@ -194,8 +187,8 @@ def test_update_state_machine():
 def test_state_machine_list_returns_empty_list_by_default():
     client = boto3.client("stepfunctions", region_name=region)
     #
-    list = client.list_state_machines()
-    list["stateMachines"].should.be.empty
+    sm_list = client.list_state_machines()
+    sm_list["stateMachines"].should.be.empty
 
 
 @mock_stepfunctions
@@ -212,25 +205,26 @@ def test_state_machine_list_returns_created_state_machines():
     machine2 = client.create_state_machine(
         name="name2", definition=str(simple_definition), roleArn=_get_default_role()
     )
-    list = client.list_state_machines()
+    sm_list = client.list_state_machines()
     #
-    list["ResponseMetadata"]["HTTPStatusCode"].should.equal(200)
-    list["stateMachines"].should.have.length_of(2)
-    list["stateMachines"][0]["creationDate"].should.be.a(datetime)
-    list["stateMachines"][0]["creationDate"].should.equal(machine1["creationDate"])
-    list["stateMachines"][0]["name"].should.equal("name1")
-    list["stateMachines"][0]["stateMachineArn"].should.equal(
+    sm_list["ResponseMetadata"]["HTTPStatusCode"].should.equal(200)
+    sm_list["stateMachines"].should.have.length_of(2)
+    sm_list["stateMachines"][0]["creationDate"].should.be.a(datetime)
+    sm_list["stateMachines"][0]["creationDate"].should.equal(machine1["creationDate"])
+    sm_list["stateMachines"][0]["name"].should.equal("name1")
+    sm_list["stateMachines"][0]["stateMachineArn"].should.equal(
         machine1["stateMachineArn"]
     )
-    list["stateMachines"][1]["creationDate"].should.be.a(datetime)
-    list["stateMachines"][1]["creationDate"].should.equal(machine2["creationDate"])
-    list["stateMachines"][1]["name"].should.equal("name2")
-    list["stateMachines"][1]["stateMachineArn"].should.equal(
+    sm_list["stateMachines"][1]["creationDate"].should.be.a(datetime)
+    sm_list["stateMachines"][1]["creationDate"].should.equal(machine2["creationDate"])
+    sm_list["stateMachines"][1]["name"].should.equal("name2")
+    sm_list["stateMachines"][1]["stateMachineArn"].should.equal(
         machine2["stateMachineArn"]
     )
 
 
 @mock_stepfunctions
+@mock_sts
 def test_state_machine_list_pagination():
     client = boto3.client("stepfunctions", region_name=region)
     for i in range(25):
@@ -928,6 +922,14 @@ def test_state_machine_get_execution_history_contains_expected_success_events_wh
     execution_history["events"].should.equal(expected_events)
 
 
+@pytest.mark.parametrize("region", ["us-west-2", "cn-northwest-1"])
+@mock_stepfunctions
+def test_stepfunction_regions(region):
+    client = boto3.client("stepfunctions", region_name=region)
+    resp = client.list_state_machines()
+    resp["ResponseMetadata"]["HTTPStatusCode"].should.equal(200)
+
+
 @mock_stepfunctions
 @mock_sts
 @mock.patch.dict(os.environ, {"SF_EXECUTION_HISTORY_TYPE": "FAILURE"})
